package com.pavelpetkevich.bcwork.activity;

import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.DefaultItemAnimator;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.pavelpetkevich.bcwork.R;
import com.pavelpetkevich.bcwork.adapters.RecyclerViewInfoTripAdapter;
import com.pavelpetkevich.bcwork.auxiliaries.Functions;
import com.pavelpetkevich.bcwork.data.ItemCarRecyclerView;
import com.pavelpetkevich.bcwork.data.ItemMyTripRecyclerView;
import com.pavelpetkevich.bcwork.dialogs.ShowDialog;
import com.pavelpetkevich.bcwork.googleMapsApi.data.CountryOnWay;
import com.pavelpetkevich.bcwork.sql.DatabaseHelper;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;

public class DetailTripActivity extends AppCompatActivity {

    @BindView(R.id.originTxtView)
    TextView originTxtView;
    @BindView(R.id.destinationTxtView)
    TextView destinationTxtView;
    @BindView(R.id.distanceTxtView)
    TextView distanceTxtView;

    @BindView(R.id.txtViewTotalTollCost)
    TextView txtViewTotalTollCost;
    @BindView(R.id.txtViewTotalFuelCost)
    TextView txtViewTotalFuelCost;
    @BindView(R.id.txtViewTotalAll)
    TextView txtViewTotalAll;

    @BindView(R.id.imgSomeWrong)
    ImageView imgSomeWrong;
    @BindView(R.id.tv_error_title_primary)
    TextView tv_error_title_primary;
    @BindView(R.id.tv_error_message_secondary)
    TextView tv_error_message_secondary;

    @BindView(R.id.my_recycler_view)
    RecyclerView recyclerView;

    private RecyclerViewInfoTripAdapter adapter;
    private String duration;
    private DatabaseHelper databaseHelper;

    private List<CountryOnWay> countryOnWays = new ArrayList<>();

    private ItemMyTripRecyclerView itemMyTripRecyclerView;
    private ItemCarRecyclerView myCar;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_detail_trip);
        ButterKnife.bind(this);
        itemMyTripRecyclerView = new ItemMyTripRecyclerView();

        getBundleExtras();
        setRecyclerView();
        setContentViewComponents();
        postDataToSQLite();
    }

    /**
     * Get extras from MapsActivity
     */
    private void getBundleExtras() {
        Bundle extras = getIntent().getExtras();
        if (extras != null) {
            duration = extras.getString("duration");
            originTxtView.setText(extras.getString("origin"));
            destinationTxtView.setText(extras.getString("destination"));
            distanceTxtView.setText(extras.getString("distance"));
            countryOnWays = extras.getBundle("bundle").getParcelableArrayList("countriesResult");
            myCar = extras.getBundle("bundle").getParcelable("myCar");
        }
    }

    /**
     * Set RecyclerView: Size, Margins and so on.
     */
    private void setRecyclerView() {
        assert myCar != null;
        adapter = new RecyclerViewInfoTripAdapter((ArrayList<CountryOnWay>) countryOnWays, this, String.valueOf(myCar.getConsummation()));
        recyclerView.setHasFixedSize(true);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));
        recyclerView.setItemAnimator(new DefaultItemAnimator());
        recyclerView.setAdapter(adapter);
    }

    /**
     * Set components:  Total_Cost, Total_Fuel, Total_All.
     */
    private void setContentViewComponents() {
        txtViewTotalTollCost.setText(adapter.getSumTollCost() + "");
        txtViewTotalFuelCost.setText(adapter.getSumFuelCost() + "");
        txtViewTotalAll.setText(adapter.getSumTollCost() + adapter.getSumFuelCost() + "");
    }

    /**
     * Post dates to SQLite DB by methods controlDatesBeforePosting() && setAddTripDB();
     */
    private void postDataToSQLite() {
        databaseHelper = new DatabaseHelper(this);
        if (controlDatesBeforePosting()) {
            setAddTripDB();
            showDetails();
        } else {
            showErrorMessage("Unknown problem", "In some problem dates aren't saved, try again later.\nOr contact us, thanks.");
        }
    }

    /**
     * Only set object and call method from DatabaseHelper to post dates to SQLite DB.
     */
    private void setAddTripDB() {
        itemMyTripRecyclerView = new ItemMyTripRecyclerView(Functions.randID(), originTxtView.getText().toString(),
                destinationTxtView.getText().toString(), distanceTxtView.getText().toString(), duration, txtViewTotalAll.getText().toString(), myCar.getId(), false);
        databaseHelper.addTrip(itemMyTripRecyclerView);
    }

    /**
     * Control dates before post to SQLite DB.
     *
     * @return true if all is good.
     */
    private boolean controlDatesBeforePosting() {
        return (!originTxtView.getText().equals("")
                && !destinationTxtView.getText().equals("")
                && !distanceTxtView.getText().equals("")
                && !txtViewTotalAll.getText().equals("")
                && !duration.equals("")
                && myCar != null);
    }

    /**
     * This method show's Error Message and un shown RecyclerView when the list is empty.
     *
     * @param title - error title
     * @param msg   - error message
     */
    private void showErrorMessage(String title, String msg) {
        recyclerView.setVisibility(View.INVISIBLE);

        tv_error_title_primary.setVisibility(View.VISIBLE);
        tv_error_message_secondary.setVisibility(View.VISIBLE);
        imgSomeWrong.setVisibility(View.VISIBLE);

        imgSomeWrong.setImageResource(R.drawable.ic_error_uknown);
        tv_error_title_primary.setText(title);
        tv_error_message_secondary.setText(msg);
    }

    /**
     * Show details of trip if all is good
     */
    private void showDetails() {
        tv_error_title_primary.setVisibility(View.INVISIBLE);
        tv_error_message_secondary.setVisibility(View.INVISIBLE);
        imgSomeWrong.setVisibility(View.INVISIBLE);

        recyclerView.setVisibility(View.VISIBLE);
        ShowDialog.showProgressDialog(false, getString(R.string.progress_dialog_places_text), this);
    }
}
