package com.pavelpetkevich.bcwork.activity;

import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.BottomNavigationView;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;

import com.pavelpetkevich.bcwork.R;
import com.pavelpetkevich.bcwork.fragment.HistoryFragment;
import com.pavelpetkevich.bcwork.fragment.HomeFragment;
import com.pavelpetkevich.bcwork.fragment.PlacesFragment;

import butterknife.BindView;
import butterknife.ButterKnife;

public class MainActivity extends AppCompatActivity {

    @BindView(R.id.appToolbarMain)
    Toolbar appToolbarMain;
    @BindView(R.id.navigation)
    BottomNavigationView bottomNavigationView;

    private final static String HOME_FRAG = "home";
    private final static String HISTORY_FRAG = "history";
    private final static String PLACES_FRAG = "places";

    private boolean setNavItemOnHistory = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        ButterKnife.bind(this);
        setSupportActionBar(appToolbarMain);
        callFragmentByName(HOME_FRAG);
        bottomNavigationView.setOnNavigationItemSelectedListener(mOnNavigationItemSelectedListener);
        getBundleExtras();
        if (setNavItemOnHistory) {
            bottomNavigationView.setSelectedItemId(R.id.navigation_history);
        }
    }

    private void getBundleExtras() {
        Bundle extras = getIntent().getExtras();
        if (extras != null) {
            if (extras.getBundle("bundle") != null) {
                setNavItemOnHistory = extras.getBundle("bundle").getBoolean("history");
            }
        }

    }

    /**
     * Navigation View Item Selected Listener
     */
    private BottomNavigationView.OnNavigationItemSelectedListener mOnNavigationItemSelectedListener
            = new BottomNavigationView.OnNavigationItemSelectedListener() {

        @Override
        public boolean onNavigationItemSelected(@NonNull MenuItem item) {
            switch (item.getItemId()) {
                case R.id.navigation_home:
                    callFragmentByName(HOME_FRAG);
                    return true;
                case R.id.navigation_history:
                    callFragmentByName(HISTORY_FRAG);
                    return true;
                case R.id.navigation_places:
                    callFragmentByName(PLACES_FRAG);
                    return true;
            }
            return false;
        }
    };

    /**
     * Call fragments by name
     *
     * @param name - name of fragment
     */
    private void callFragmentByName(String name) {
        FragmentManager fragmentManager = getSupportFragmentManager();
        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
        switch (name) {
            case HOME_FRAG:
                HomeFragment homeFragment = new HomeFragment();
                fragmentTransaction.replace(R.id.contentMain, homeFragment);
                fragmentTransaction.commit();
                break;
            case HISTORY_FRAG:
                HistoryFragment historyFragment = new HistoryFragment();
                fragmentTransaction.replace(R.id.contentMain, historyFragment);
                fragmentTransaction.commit();
                break;
            case PLACES_FRAG:
                PlacesFragment placesFragment = new PlacesFragment();
                fragmentTransaction.replace(R.id.contentMain, placesFragment);
                fragmentTransaction.commit();
                break;
        }
    }
}
