package com.pavelpetkevich.bcwork.activity;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.location.Location;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.os.Parcelable;
import android.support.annotation.NonNull;
import android.support.annotation.RequiresApi;
import android.support.design.widget.TextInputEditText;
import android.support.v4.app.FragmentActivity;
import android.support.v4.content.ContextCompat;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnMyLocationChangeListener;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.Polyline;
import com.google.android.gms.maps.model.PolylineOptions;
import com.google.maps.android.PolyUtil;
import com.pavelpetkevich.bcwork.R;
import com.pavelpetkevich.bcwork.auxiliaries.Functions;
import com.pavelpetkevich.bcwork.data.ItemCarRecyclerView;
import com.pavelpetkevich.bcwork.dialogs.ShowDialog;
import com.pavelpetkevich.bcwork.googleMapsApi.data.CountriesList;
import com.pavelpetkevich.bcwork.googleMapsApi.data.CountryOnWay;
import com.pavelpetkevich.bcwork.googleMapsApi.data.DirectionResults;
import com.pavelpetkevich.bcwork.googleMapsApi.data.Legs;
import com.pavelpetkevich.bcwork.googleMapsApi.data.OverviewPolyline;
import com.pavelpetkevich.bcwork.googleMapsApi.data.Route;
import com.pavelpetkevich.bcwork.googleMapsApi.data.Steps;
import com.pavelpetkevich.bcwork.googleMapsApi.service.GoogleApiService;
import com.pavelpetkevich.bcwork.googleMapsApi.service.GoogleRetroClient;
import com.pavelpetkevich.bcwork.service.ApiService;
import com.pavelpetkevich.bcwork.service.ControlServices;
import com.pavelpetkevich.bcwork.service.RetroClient;
import com.pavelpetkevich.bcwork.sql.DatabaseHelper;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class MapsActivity extends FragmentActivity implements OnMapReadyCallback, GoogleMap.OnMyLocationButtonClickListener {

    private static final String COUNTRIES_COSTS_URL = "https://api.myjson.com";

    @BindView(R.id.infoTripBtn)
    Button btnMoreInfo;
    @BindView(R.id.originAddressInEdTxt)
    TextInputEditText originAddressInEdTxt;
    @BindView(R.id.destinationAddressInEdTxt)
    TextInputEditText destinationAddressInEdTxt;

    @BindView(R.id.txtDistance)
    TextView txtDistance;
    @BindView(R.id.txtTime)
    TextView txtTime;

    private List<Marker> originMarkers = new ArrayList<>();
    private List<Marker> destinationMarkers = new ArrayList<>();
    private List<Polyline> polylinePaths = new ArrayList<>();

    private List<CountryOnWay> countriesResult;
    private List<CountryOnWay> countryOnWays;
    private LatLng start, finish;

    private ItemCarRecyclerView myCar;
    private List<ItemCarRecyclerView> itemCarRecyclerViews;
    private DatabaseHelper databaseHelper;

    private boolean currentLocationIsGet = false;
    private boolean carIsChoose = false;
    private boolean showDialogResponse = false;

    private GoogleMap mMap;
    private String origin, destination;

    private Activity mActivity;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_maps);
        ButterKnife.bind(this);

        countriesResult = new ArrayList<>();
        countryOnWays = new ArrayList<>();

        itemCarRecyclerViews = new ArrayList<>();
        databaseHelper = new DatabaseHelper(this);

        getDataAboutCarsFromSQLite();

        SupportMapFragment mapFragment = (SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map);
        mapFragment.getMapAsync(this);
    }

    @Override
    protected void onResume() {
        super.onResume();
        mActivity = this;
        getDataAboutCarsFromSQLite();
    }

    @Override
    protected void onPause() {
        super.onPause();
        //Functions.clearAllEdts(originAddressInEdTxt);
        //Functions.clearAllEdts(destinationAddressInEdTxt);
        //txtDistance.setText("0 km");
        //txtTime.setText("0 min");
        //removeAllMarkersPoly();
        cleanLists();
    }

    @OnClick(R.id.makeTripBtn)
    public void onClickMakeTripBtn(View view) {
        if (carIsChoose && myCar != null) {
            //After take car push data to different class which calculate fuel!
            if (edtNoEmpty())
                sendRequest();
        } else {
            if (itemCarRecyclerViews.isEmpty()) {
                Intent intent = new Intent(this, MyCarsActivity.class);
                startActivity(intent);
            } else {
                Dialog dialog = onCreateDialogSingleCarChoice();
                dialog.show();
                ((AlertDialog) dialog).getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(false);
                ListView lw = ((AlertDialog) dialog).getListView();
                ((AlertDialog) dialog).getListView().setItemChecked(lw.getCheckedItemPosition(), false);
            }
        }
    }

    @OnClick(R.id.navigateBtn)
    public void onClickNavigateBtn(View view) {
        openGoogleMaps();
    }

    @OnClick(R.id.infoTripBtn)
    public void onClickInfoTripBtn(View view) {
        callDetailTripActivity();
    }

    //Work with map an current location
    @SuppressLint("MissingPermission")
    @Override
    public void onMapReady(GoogleMap googleMap) {
        mMap = googleMap;
        ShowDialog.showDialogDecideGetLocation(mMap, myLocationChangeListener, mActivity);
        mMap.setMyLocationEnabled(true);

        mMap.setOnMyLocationButtonClickListener(this);
        mMap.getUiSettings().setMapToolbarEnabled(false);
        mMap.getUiSettings().setCompassEnabled(true);
        mMap.getUiSettings().setZoomControlsEnabled(true);
    }

    @Override
    public boolean onMyLocationButtonClick() {
        if (originMarkers != null) {
            for (Marker marker : originMarkers) {
                marker.remove();
            }
        }
        if (mMap.getMyLocation() != null) {
            LatLng loc = new LatLng(mMap.getMyLocation().getLatitude(), mMap.getMyLocation().getLongitude());
            originAddressInEdTxt.setText(Functions.getAddressFromLatLng(loc.latitude, loc.longitude, this));
            originMarkers.add(mMap.addMarker(new MarkerOptions()
                    .position(new MarkerOptions().position(loc).getPosition())
                    .title(getString(R.string.current_location))
                    .icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_RED))
                    .alpha(0.7f)
                    .snippet(loc.latitude + ";" + loc.longitude)
                    .flat(true)
            ));
            if (mMap != null) {
                mMap.animateCamera(CameraUpdateFactory.newLatLngZoom(loc, 16.0f));
            }
        } else {
            ShowDialog.showErrorDialog(getString(R.string.current_location), getString(R.string.current_location_error), mActivity);
        }
        return false;
    }

    /**
     * myLocationChangeListener - this method is used for take current location.
     */
    private OnMyLocationChangeListener myLocationChangeListener;

    {
        myLocationChangeListener = new OnMyLocationChangeListener() {
            @Override
            public void onMyLocationChange(Location location) {
                if (!currentLocationIsGet) {
                    if (originMarkers != null) {
                        for (Marker marker : originMarkers) {
                            marker.remove();
                        }
                    }
                    LatLng loc = new LatLng(location.getLatitude(), location.getLongitude());
                    originAddressInEdTxt.setText(Functions.getAddressFromLatLng(loc.latitude, loc.longitude, mActivity));
                    originMarkers.add(mMap.addMarker(new MarkerOptions()
                            .position(new MarkerOptions().position(loc).getPosition())
                            .title(getString(R.string.current_location))
                            .icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_RED))
                            .alpha(0.7f)
                            .snippet(loc.latitude + ";" + loc.longitude)
                            .flat(true)
                    ));
                    if (mMap != null) {
                        mMap.animateCamera(CameraUpdateFactory.newLatLngZoom(loc, 16.0f));
                    }
                    currentLocationIsGet = true;
                }
            }
        };
    }

    //Branch from here

    /**
     * This is the main method of creating a trip.
     * Send request to google maps api and use all methods belows...
     * Some comments about functionality, i've inside of this method, because the method is so huge,
     * to rewrite functionality here.
     */
    private void sendRequest() {
        cleanLists();
        ShowDialog.showProgressDialog(true, "Make your trip, wait a moment", mActivity);
        if (edtNoEmpty()) {
            if (ControlServices.controlNetworkStateService(mActivity) && ControlServices.controlGpsProviderServiceIsAvailable(mActivity)) {
                GoogleApiService api = GoogleRetroClient.getApiService();
                api.getJson(origin, destination).enqueue(new Callback<DirectionResults>() {
                    //This is the main part o this method if onResponse is failed called onFailure
                    @RequiresApi(api = Build.VERSION_CODES.M)
                    @Override
                    public void onResponse(@NonNull Call<DirectionResults> call, @NonNull Response<DirectionResults> response) {
                        /*
                          Get all what we need:
                            route, legs, steppes, overviewPolyline.
                            We needed that for create trip.
                         */
                        if (response.body().getRoutes().size() != 0) {
                            Route route = response.body().getRoutes().get(0);
                            Legs legs = route.getLegsList().get(0);
                            List<Steps> steppes = legs.getStepses();
                            OverviewPolyline overviewPolyline = route.getOverviewPolyline();

                            txtDistance.setText(legs.getDistance().getText());
                            txtTime.setText(legs.getDuration().getText());

                            CountryOnWay obj;
                            LatLng countryBoardStart;
                            int idCountry = 0;
                            //Go throw all steppes
                            for (int j = 0; j < steppes.size(); j++) {
                                countryBoardStart = new LatLng(steppes.get(j).getLatLonCoordinatesEnd().getLat(), steppes.get(j).getLatLonCoordinatesEnd().getLng());
                                //if it's first/last step, we didn't used html_instructions.
                                if (steppes.size() - 1 == j || j == 0) {
                                    if (j == 0) {
                                        String countryName = getCountryNameFromAddress(legs.getStartAddress());
                                        obj = createCountry(idCountry, countryName, countryBoardStart, "0", "0", "0", "0");
                                        countryOnWays.add(obj);
                                        getCountriesCostsApi(countryName, obj);
                                        idCountry++;
                                    } else {
                                        countryOnWays.add(createCountry(idCountry, destination, countryBoardStart, "0", "n/a", "0", "0"));
                                        idCountry++;
                                    }
                                }
                                //if it's not first/last step we used html_instructions, for get countryName.
                                String countryName = splitHtmlInstructionsCountry(steppes.get(j).getHtml_instructions());
                                if (!countryName.isEmpty()) {
                                    obj = createCountry(idCountry, countryName, countryBoardStart, "0", "0", "0", "");
                                    Log.e("obj:", obj.getCountry()+","+ obj.getCountryBoardStart() + "");
                                    countryOnWays.add(obj);
                                    getCountriesCostsApi(countryName, obj);
                                    idCountry++;
                                }
                            }

                            showDialogResponse = true;

                            //This part of code is for calculate distance between countries if them are more than one.
                            for (int i = 0; i < countryOnWays.size(); i++) {
                                setStartFinishPoints(i, countryOnWays);
                                final int finalI = i;
                                GoogleApiService apiService = GoogleRetroClient.getApiService();
                                apiService.getJson(start.latitude + "," + start.longitude, finish.latitude + "," + finish.longitude).enqueue(new Callback<DirectionResults>() {
                                    @Override
                                    public void onResponse(@NonNull Call<DirectionResults> call, @NonNull Response<DirectionResults> response) {
                                        if (response.body().getRoutes().size() != 0) {
                                            Route route = response.body().getRoutes().get(0);
                                            Legs legs = route.getLegsList().get(0);
                                            setCountryDistance(finalI, countryOnWays, legs);
                                        }
                                        showDialogResponse = true;
                                    }

                                    @Override
                                    public void onFailure(Call<DirectionResults> call, Throwable t) {
                                        showDialogResponse = false;
                                        ShowDialog.showProgressDialog(false, getString(R.string.make_trip_msg), mActivity);
                                        ShowDialog.showErrorDialog(getString(R.string.error_calculate_distance), getString(R.string.error_calculate_distance_msg), mActivity);

                                        Log.e("onFailure", "createPathsRoute: " + t);
                                    }
                                });

                                if (countryOnWays.size() - 1 != i)
                                    countriesResult.add(i, countryOnWays.get(i));
                            }

                            //Final part create poly, markers and set edit texts, but first of all removeAll.
                            removeAllMarkersPoly();
                            setPolylinePaths(response.body().getRoutes(), legs, PolyUtil.decode(overviewPolyline.getPoints()));

                            if (showDialogResponse) {
                                ShowDialog.showDecideDialog(getString(R.string.success_create_trip), getString(R.string.success_create_trip_msg), mActivity, new ShowDialog.DialogButtonListener() {
                                    @Override
                                    public void onButtonClicked(boolean value) {
                                        if (value) {
                                            callDetailTripActivity();
                                        } else {
                                            btnMoreInfo.setFocusable(true);
                                            btnMoreInfo.setFocusableInTouchMode(true);
                                            btnMoreInfo.requestFocus();
                                        }
                                    }
                                });
                            }
                        } else {
                            ShowDialog.showProgressDialog(false, getString(R.string.make_trip_msg), mActivity);
                            ShowDialog.showErrorDialog(getString(R.string.error_no_find_routes), getString(R.string.error_no_find_routes_msg), mActivity);
                        }
                        ShowDialog.showProgressDialog(false, getString(R.string.make_trip_msg), mActivity);
                    }

                    @Override
                    public void onFailure(@NonNull Call<DirectionResults> call, @NonNull Throwable t) {
                        ShowDialog.showProgressDialog(false, getString(R.string.make_trip_msg), mActivity);
                        ShowDialog.showErrorDialog(getString(R.string.error_make_trip), getString(R.string.error_make_trip_msg), mActivity);
                        Log.e("sendRequest:OnFailure", "Error:" + t.getMessage());
                    }
                });

            }
        }
    }

    /**
     * This method is used for take costs from api.
     * How it's works:
     * 1) Connect to api.
     * 2) In enqueue try take dates, which will be saved in "obj" object.
     * 3) Compare to countries if are equals, setToll_cost and setComment.
     * 4) TODO Make FUEL_COST set.
     *
     * @param countryName - country name
     * @param obj         - country object
     */
    private void getCountriesCostsApi(final String countryName, final CountryOnWay obj) {
        ApiService apiService = RetroClient.getApiService(COUNTRIES_COSTS_URL);
        final String finalStr = countryName.trim();
        apiService.getCountriesCosts().enqueue(new Callback<CountriesList>() {
            @Override
            public void onResponse(@NonNull Call<CountriesList> call, @NonNull Response<CountriesList> response) {
                List<CountriesList.Country> countryDates = response.body().getCountry();
                for (int i = 0; i < countryDates.size(); i++) {
                    String country = countryDates.get(i).getCountryDates().getNameCountry();
                    country = country.trim();
                    if (country.equals(finalStr)) {
                        obj.setToll_cost(countryDates.get(i).getCountryDates().getToll_cost());
                        obj.setComment(countryDates.get(i).getCountryDates().getComment());
                        obj.setFuel_cost(countryDates.get(i).getCountryDates().getFuelCost().getNatural());

                        Log.e("Country", country + ", Type of toll:" + countryDates.get(i).getCountryDates().getType_toll()
                                + ", Toll sum:" + countryDates.get(i).getCountryDates().getToll_cost() + ", Comment:" + countryDates.get(i).getCountryDates().getComment());
                    }
                }
            }

            @Override
            public void onFailure(@NonNull Call<CountriesList> call, @NonNull Throwable t) {
                ShowDialog.showErrorDialog(getString(R.string.error_costs), getString(R.string.error_costs_msg), mActivity);
                Log.e("onFailure", "getCountriesCostsApi: " + t);
            }
        });
    }

    /**
     * This method is for create object of country.
     *
     * @param idCountry         - country id.
     * @param countryName       - country name.
     * @param countryBoardStart - country start on a board.
     * @param distance          - country distance(how much km on this country).
     * @param comment           - country comment.
     * @param toll_cost         - country toll costs.
     * @return country object
     */
    private CountryOnWay createCountry(int idCountry, String countryName, LatLng countryBoardStart, String distance, String comment, String toll_cost, String fuel_cost) {
        return new CountryOnWay(idCountry, countryName, countryBoardStart, distance, comment, toll_cost, fuel_cost);
    }

    /**
     * This method is for set distance for country.get(i).
     *
     * @param i         - is a position in List(CountyOnMyWay).
     * @param countries - List of countries.
     * @param legs      - contains steppes, end,start_location,end,start_address and so on.
     */
    private static void setCountryDistance(int i, List<CountryOnWay> countries, Legs legs) {
        if (!countries.isEmpty()) {
            if (countries.size() - 1 != i) {
                String numberOnly = legs.getDistance().getText().replaceAll("[,km]", "");
                countries.get(i).setDistance(numberOnly);
                Log.e("Log.e" + i, "Distance: " + numberOnly);
            }
        }
    }

    /**
     * This method is for set ours start and end latitude, longitude.
     *
     * @param i         - is a position in List(CountyOnMyWay)
     * @param countries - List of countries
     */
    private void setStartFinishPoints(int i, List<CountryOnWay> countries) {
        if (countries.size() - 1 != i) {
            start = getStartFinishLatLng(i, countries);
            finish = getStartFinishLatLng(i + 1, countries);
        } else {
            start = getStartFinishLatLng(i - 1, countries);
            finish = getStartFinishLatLng(i, countries);
        }
    }

    /**
     * This method is for get latLng from List(CountyOnMyWay) for calculate distance
     *
     * @param i       - is a position in List(CountyOnMyWay)
     * @param country - List of countries
     * @return latitude, longitude.
     */
    private static LatLng getStartFinishLatLng(int i, List<CountryOnWay> country) {
        return new LatLng(country.get(i).getCountryBoardStart().latitude, country.get(i).getCountryBoardStart().longitude);
    }

    /**
     * How it's work:
     * 1) The regex [".*,\\s*(.*)"] -> which is used for take word after last coma.
     * 2) If m.find() -> take from m.group(1) country name and return it.
     *
     * @param str - address from which we try to take only country name.
     * @return country name.
     */
    private String getCountryNameFromAddress(String str) {
        Pattern p = Pattern.compile(".*,\\s*(.*)");
        Matcher m = p.matcher(str);
        if (m.find()) {
            str = m.group(1);
        }
        return str;
    }

    /**
     * Method for take country from html_instructions.
     * How it works:
     * 1) Take strMain and replace all '<' on '>', why? because after that we simply split -> >Entering Germany>.
     * 2) Split strMain(>Entering Germany>) by '>' and get Entering Germany
     * 3) Convert strArray to strList, because we need to delete one word "Entering".
     * 4) If strArray didn't have "Entering" delete it, this str doesn't matter for us.
     * 5) If strArray have "Entering", split the last part of str and delete "Entering".
     * 6) In the end append to StringBuilder result. Return result.
     *
     * @param strMain - steppes.get(j).getHtml_instructions() -> main HTML_INSTRUCTIONS from JSON API.
     * @return country name.
     */
    private String splitHtmlInstructionsCountry(String strMain) {
        String result;
        StringBuilder stringBuilder = new StringBuilder();
        strMain = strMain.replaceAll("<", ">");
        String[] strArray = strMain.split(">");
        List<String> strList = new ArrayList<>(Arrays.asList(strArray));
        for (int i = 0; i < strArray.length; i++) {
            if (!strArray[i].contains("Entering")) {
                strList.remove(strArray[i]);
            } else {
                strMain = strArray[i];
                strArray = strMain.split(" ");
                strList = new ArrayList<>(Arrays.asList(strArray));
                for (String aStrArray : strArray) {
                    if (aStrArray.contains("Entering")) {
                        strList.remove(aStrArray);
                    } else {
                        stringBuilder.append(aStrArray).append(" ");
                    }
                }
            }
        }
        result = stringBuilder.toString();
        return result;
    }

    //--------------------------------------------------------------------------------------------------------------------------------------------

    /**
     * Get dates about cars from SQLite DB.
     * After getAllCars input them to the itemCarRecyclerViews(List(ItemCarRecyclerView)).
     * AsyncTask is used that SQLite operation not blocks the UI Thread.
     */
    @SuppressLint("StaticFieldLeak")
    private void getDataAboutCarsFromSQLite() {
        new AsyncTask<Void, Void, Void>() {
            @Override
            protected Void doInBackground(Void... params) {
                itemCarRecyclerViews.clear();
                itemCarRecyclerViews.addAll(databaseHelper.getAllCars());
                return null;
            }

            @Override
            protected void onPostExecute(Void aVoid) {
                super.onPostExecute(aVoid);
                //TODO onPostExecute SQLite Cars.
            }
        }.execute();
    }

    /**
     * Create view trip, set poly, markers, org/dest.
     * And of cause move camera.
     *
     * @param routes - contains legs, overview_polyline, summary, warnings and so on.
     * @param legs   - contains steppes, end,start_location,end,start_address and so on.
     * @param points - PolyUtil.decode(overviewPolyline.getPoints()) - decode points from overviewPolyline
     */
    private void setPolylinePaths(List<Route> routes, Legs legs, List<LatLng> points) {
        String start_address, end_address;
        LatLng start_location, end_location;

        polylinePaths = new ArrayList<>();
        originMarkers = new ArrayList<>();
        destinationMarkers = new ArrayList<>();

        start_address = legs.getStartAddress();
        end_address = legs.getEndAddress();
        start_location = new LatLng(legs.getLatLonCoordinatesStart().getLat(), legs.getLatLonCoordinatesStart().getLng());
        end_location = new LatLng(legs.getLatLonCoordinatesEnd().getLat(), legs.getLatLonCoordinatesEnd().getLng());

        for (Route route : routes) {
            mMap.moveCamera(CameraUpdateFactory.newLatLngZoom(start_location, 10));
            addMarkersPolySetEdt(start_address, end_address, start_location, end_location, points);
        }
    }

    /**
     * This method add markers, set edit texts and create poly line.
     *
     * @param start_address  - origin.
     * @param end_address    - destination.
     * @param start_location - for create marker origin.
     * @param end_location   - for create marker destination.
     * @param points         - it's all points for create poly line.
     */
    private void addMarkersPolySetEdt(String start_address, String end_address, LatLng start_location,
                                      LatLng end_location, List<LatLng> points) {
        originAddressInEdTxt.setText(start_address);
        destinationAddressInEdTxt.setText(end_address);

        originMarkers.add(mMap.addMarker(new MarkerOptions()
                .title(start_address)
                .position(start_location)
                .icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_RED))
                .alpha(0.7f)
                .snippet(start_location.latitude + ";" + start_location.longitude)
                .flat(true)
        ));

        destinationMarkers.add(mMap.addMarker(new MarkerOptions()
                //.icon(BitmapDescriptorFactory.fromResource(R.mipmap.ic_launcher_round))
                .title(end_address)
                .position((end_location))
                .icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_RED))
                .alpha(0.7f)
                .snippet(end_location.latitude + ";" + end_location.longitude)
                .flat(true)
        ));

        PolylineOptions polylineOptions = new PolylineOptions().
                geodesic(true).
                color(ContextCompat.getColor(mActivity, R.color.colorAccent)).
                width(10);

        for (int i = 0; i < points.size(); i++)
            polylineOptions.add(points.get(i));

        polylinePaths.add(mMap.addPolyline(polylineOptions));
    }

    /**
     * Remove all markers and poly lines from map.
     */
    private void removeAllMarkersPoly() {
        if (originMarkers != null) {
            for (Marker marker : originMarkers) {
                marker.remove();
            }
        }

        if (destinationMarkers != null) {
            for (Marker marker : destinationMarkers) {
                marker.remove();
            }
        }

        if (polylinePaths != null) {
            for (Polyline polyline : polylinePaths) {
                polyline.remove();
            }
        }
    }

    /**
     * CLean all lists.
     */
    private void cleanLists() {
        if (!countriesResult.isEmpty()) countriesResult.clear();
        if (!countryOnWays.isEmpty()) countryOnWays.clear();
    }

    /**
     * If Edit texts no empty ready for create trip.
     *
     * @return true if all is ok.
     */
    private boolean edtNoEmpty() {
        origin = originAddressInEdTxt.getText().toString();
        destination = destinationAddressInEdTxt.getText().toString();
        if (origin.isEmpty()) {
            Toast.makeText(this, R.string.warning_fill_origin, Toast.LENGTH_SHORT).show();
            return false;
        }
        if (destination.isEmpty()) {
            Toast.makeText(this, R.string.warning_fill_destination, Toast.LENGTH_SHORT).show();
            return false;
        }
        return true;
    }

    /**
     * Open google maps navigation
     */
    private void openGoogleMaps() {
        if (edtNoEmpty()) {
            if (txtDistance.getText().toString().equals("0 km")) {
                Toast.makeText(this, R.string.warning_make_trip, Toast.LENGTH_SHORT).show();
                return;
            }
            String uri = "https://www.google.com/maps/dir/?api=1&origin=" + originAddressInEdTxt.getText()
                    + "&waypoints=+" + destinationAddressInEdTxt.getText() + "+&travelmode=driving&dir_action=navigate";
            Intent intent = new Intent(android.content.Intent.ACTION_VIEW, Uri.parse(uri));
            intent.setClassName("com.google.android.apps.maps", "com.google.android.maps.MapsActivity");
            startActivity(intent);
        }
    }

    /**
     * This method for create custom SingleChoice dialog.
     * This dialog isn't locate in "ShowDialog.java", because is more simpler to make here, than to create method with crowds params.
     *
     * @return dialog
     */
    public Dialog onCreateDialogSingleCarChoice() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this, R.style.AlertDialogCustomStyle);
        final CharSequence[] array = Functions.convertListToArrayCar(itemCarRecyclerViews);
        builder.setTitle(R.string.car_dialog_title)
                .setSingleChoiceItems(array, 1, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        for (int i = 0; i < itemCarRecyclerViews.size(); i++) {
                            String s2 = itemCarRecyclerViews.get(i).getNameCar() + "\nConsummation: " + itemCarRecyclerViews.get(i).getConsummation() + " (l/100km)";
                            if (array[which].toString().equals(s2)) {
                                setCarIsChoose(true);

                                myCar = new ItemCarRecyclerView(itemCarRecyclerViews.get(i).getId(),
                                        itemCarRecyclerViews.get(i).getNameCar(), itemCarRecyclerViews.get(i).getLicensePlate(),
                                        itemCarRecyclerViews.get(i).getVin(), itemCarRecyclerViews.get(i).getConsummation());

                                ((AlertDialog) dialog).getButton(AlertDialog.BUTTON_POSITIVE).setEnabled(true);

                                Log.e("Choose car", myCar.getNameCar() + ", " + carIsChoose);
                            }
                        }
                    }
                })
                .setPositiveButton(R.string.dialog_choose_button, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int id) {
                        // User clicked Choose and here is only close.
                        if (edtNoEmpty())
                            sendRequest();
                    }
                })
                .setNegativeButton(R.string.dialog_cancel_button, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int id) {
                        myCar = null;
                        setCarIsChoose(false);
                    }
                });
        return builder.create();
    }

    /**
     * If car is chosen, this parameter sad me that.
     *
     * @param carIsChoose - if chosen is it true.
     */
    public void setCarIsChoose(boolean carIsChoose) {
        this.carIsChoose = carIsChoose;
    }

    /**
     * Calling detail trip activity.
     * And pudh some dates from this activity to detail for calculating price.
     */
    private void callDetailTripActivity() {
        if (edtNoEmpty()) {
            if (txtDistance.getText().toString().equals("0 km") || countriesResult.isEmpty()) {
                Toast.makeText(this, R.string.warning_make_trip, Toast.LENGTH_SHORT).show();
                return;
            }
            Intent intent = new Intent(this, DetailTripActivity.class);
            intent.putExtra("duration", txtTime.getText());
            intent.putExtra("origin", origin);
            intent.putExtra("destination", destination);
            intent.putExtra("distance", txtDistance.getText());
            Bundle bundle = new Bundle();
            bundle.putParcelableArrayList("countriesResult", (ArrayList<? extends Parcelable>) countriesResult);
            bundle.putParcelable("myCar", myCar);
            intent.putExtra("bundle", bundle);
            overridePendingTransition(R.anim.enter, R.anim.exit);
            startActivity(intent);
        }
    }

}

