package com.pavelpetkevich.bcwork.activity;

import android.annotation.SuppressLint;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.View;

import com.pavelpetkevich.bcwork.R;
import com.pavelpetkevich.bcwork.data.ItemCarRecyclerView;
import com.pavelpetkevich.bcwork.fragment.AddEditCarFragment;
import com.pavelpetkevich.bcwork.fragment.MyCarsFragment;
import com.pavelpetkevich.bcwork.sql.DatabaseHelper;

import java.util.ArrayList;

import butterknife.BindView;
import butterknife.ButterKnife;

public class MyCarsActivity extends AppCompatActivity implements View.OnClickListener {

    private DatabaseHelper databaseHelper;
    private ArrayList<ItemCarRecyclerView> itemList;

    @BindView(R.id.appToolbarMain)
    Toolbar appToolbar;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_my_cars);
        ButterKnife.bind(this);

        setSupportActionBar(appToolbar);
        appToolbar.setNavigationIcon(R.drawable.ic_arrow_back);
        appToolbar.setNavigationOnClickListener(this);

        itemList = new ArrayList<>();
        databaseHelper = new DatabaseHelper(this);

        getDataFromSQLite();
    }

    @Override
    public void onClick(View v) {
        onBackPressed();
    }

    /**
     * Get dates form SQLite and if is no dates call AddEditCarFragment else MyCarsFragment
     */
    @SuppressLint("StaticFieldLeak")
    private void getDataFromSQLite() {
        // AsyncTask is used that SQLite operation not blocks the UI Thread.
        new AsyncTask<Void, Void, Void>() {
            @Override
            protected Void doInBackground(Void... params) {
                itemList.clear();
                itemList.addAll(databaseHelper.getAllCars());
                return null;
            }

            @Override
            protected void onPostExecute(Void aVoid) {
                super.onPostExecute(aVoid);
                if (itemList.isEmpty()) {
                    callFragment("add");
                } else {
                    callFragment("list");
                }
            }
        }.execute();
    }

    /**
     * Call fragment by name
     * If name is add - push some information to AddEditCarFragment:
     * "mainBack" if it's true mean that inside of AddEditCarFragment user can backPressed only to the Home else can back to the list.
     * "edit" if it's true mean that call for editing else call for add new item.
     *
     * @param name - is name off fragment
     */
    private void callFragment(String name) {
        FragmentManager fragmentManager = getSupportFragmentManager();
        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
        AddEditCarFragment addEditCarFragment = new AddEditCarFragment();
        MyCarsFragment myCarsFragment = new MyCarsFragment();
        switch (name) {
            case "add":
                fragmentTransaction.setCustomAnimations(R.anim.enter, R.anim.exit);
                Bundle bundle = new Bundle();
                bundle.putBoolean("mainBack", true);
                bundle.putBoolean("edit", false);
                addEditCarFragment.setArguments(bundle);
                fragmentTransaction.replace(R.id.incl_fragment, addEditCarFragment);
                fragmentTransaction.commit();
                break;
            case "list":
                fragmentTransaction.setCustomAnimations(R.anim.enter, R.anim.exit);
                fragmentTransaction.replace(R.id.incl_fragment, myCarsFragment);
                fragmentTransaction.commit();
                break;
        }
    }

}
