package com.pavelpetkevich.bcwork.adapters;

import android.content.Context;
import android.os.Bundle;
import android.support.design.widget.Snackbar;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.pavelpetkevich.bcwork.R;
import com.pavelpetkevich.bcwork.data.ItemCarRecyclerView;
import com.pavelpetkevich.bcwork.fragment.AddEditCarFragment;

import java.util.ArrayList;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;


public class RecyclerViewCarAdapter extends RecyclerView.Adapter<RecyclerViewCarAdapter.ViewHolder> {

    private ArrayList<ItemCarRecyclerView> itemCarRecyclerViewArrayList;
    private Context context;

    public RecyclerViewCarAdapter(ArrayList<ItemCarRecyclerView> itemCarRecyclerViewArrayList, Context context) {
        this.itemCarRecyclerViewArrayList = itemCarRecyclerViewArrayList;
        this.context = context;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.recycler_car_row, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {

        ImageView iconCarBrand = holder.iconCarBrand;
        iconCarBrand.setImageResource(R.mipmap.ic_launcher_round);

        TextView txtCarName = holder.txtCarName;
        txtCarName.setText(itemCarRecyclerViewArrayList.get(position).getNameCar());

        TextView txtCarConsummation = holder.txtCarConsummation;
        String finalResult = Double.valueOf(itemCarRecyclerViewArrayList.get(position).getConsummation()).toString();
        txtCarConsummation.setText(finalResult);

        holder.setContext(context);
        holder.setId(itemCarRecyclerViewArrayList.get(position).getId());
        holder.setLicensePlate(itemCarRecyclerViewArrayList.get(position).getLicensePlate());
        holder.setVinNumber(itemCarRecyclerViewArrayList.get(position).getVin());
    }

    @Override
    public int getItemCount() {
        return itemCarRecyclerViewArrayList == null ? 0 : itemCarRecyclerViewArrayList.size();
    }

    static class ViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {
        private Context context;
        private String vinNumber, licensePlate;
        private int id;
        @BindView(R.id.icCarBrand)
        ImageView iconCarBrand;
        @BindView(R.id.txtCarName)
        TextView txtCarName;
        @BindView(R.id.txtCarConsummation)
        TextView txtCarConsummation;

        ViewHolder(View itemView) {
            super(itemView);
            ButterKnife.bind(this, itemView);
            itemView.setOnClickListener(this);
        }

        @Override
        public void onClick(View v) {
            callAddEditFragment();
        }

        @OnClick(R.id.btnFavoriteCar)
        void onClickFavor(View v) {
            Snackbar.make(v, "Action is pressed",
                    Snackbar.LENGTH_LONG).show();
        }


        public void setVinNumber(String vinNumber) {
            this.vinNumber = vinNumber;
        }

        public void setLicensePlate(String licensePlate) {
            this.licensePlate = licensePlate;
        }

        public void setId(int id) {
            this.id = id;
        }

        void setContext(Context context) {
            this.context = context;
        }

        private void callAddEditFragment() {
            FragmentManager fragmentManager = ((AppCompatActivity) context).getSupportFragmentManager();
            FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
            AddEditCarFragment addEditCarFragment = new AddEditCarFragment();
            fragmentTransaction.setCustomAnimations(R.anim.enter, R.anim.exit);
            Bundle bundle = new Bundle();
            bundle.putBoolean("mainBack", false);
            bundle.putBoolean("edit", true);
            bundle.putInt("_ID", id);
            bundle.putString("carName", txtCarName.getText().toString());
            bundle.putString("carCons", txtCarConsummation.getText().toString());
            bundle.putString("carLP", licensePlate);
            bundle.putString("carVin", vinNumber);
            addEditCarFragment.setArguments(bundle);
            fragmentTransaction.replace(R.id.incl_fragment, addEditCarFragment);
            //fragmentTransaction.remove(myCarsFragment);
            fragmentTransaction.commit();
        }
    }
}
