package com.pavelpetkevich.bcwork.adapters;

import android.content.Context;
import android.os.Build;
import android.os.CountDownTimer;
import android.support.annotation.RequiresApi;
import android.support.design.widget.Snackbar;
import android.support.v4.content.ContextCompat;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.util.SparseBooleanArray;
import android.view.HapticFeedbackConstants;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.daimajia.swipe.SwipeLayout;
import com.pavelpetkevich.bcwork.R;
import com.pavelpetkevich.bcwork.auxiliaries.Functions;
import com.pavelpetkevich.bcwork.data.ItemMyTripRecyclerView;
import com.pavelpetkevich.bcwork.sql.DatabaseHelper;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

/**
 * Created by pavelpetkevich on 14/11/2017.
 */

public class RecyclerViewHistoryAdapter extends RecyclerView.Adapter<RecyclerViewHistoryAdapter.ViewHolder> {
    private ArrayList<ItemMyTripRecyclerView> itemMyTripRecyclerViews;
    private Context context;

    private RecyclerViewHistoryAdapterListener listener;

    private SparseBooleanArray selectedItems;

    // array used to perform multiple animation at once
    private SparseBooleanArray animationItemsIndex;
    private boolean reverseAllAnimations = false;

    // for animate only thr selected row.
    private static int currentSelectedIndex = -1;

    private SwipeLayout mainSwipeLayout;

    private CountDownTimer waitTimer;

    public RecyclerViewHistoryAdapter(ArrayList<ItemMyTripRecyclerView> itemInfoTripRecyclerViewArrayList, Context context, RecyclerViewHistoryAdapterListener listener) {
        this.itemMyTripRecyclerViews = itemInfoTripRecyclerViewArrayList;
        this.context = context;
        this.listener = listener;

        selectedItems = new SparseBooleanArray();
        animationItemsIndex = new SparseBooleanArray();
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.recycler_view_history_row, parent, false);
        sortFavorite();
        return new RecyclerViewHistoryAdapter.ViewHolder(view);
    }

    @Override
    public int getItemCount() {
        return itemMyTripRecyclerViews == null ? 0 : itemMyTripRecyclerViews.size();
    }

    public void sortFavorite() {
        Collections.sort(itemMyTripRecyclerViews, ItemMyTripRecyclerView.itemMyTripRecyclerViewComparator);
    }

    @RequiresApi(api = Build.VERSION_CODES.N)
    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        TextView rvOriginTxtView = holder.rvOriginTxtView;
        rvOriginTxtView.setText(itemMyTripRecyclerViews.get(position).getOrigin());
        TextView rvDestinationTxtView = holder.rvDestinationTxtView;
        rvDestinationTxtView.setText(itemMyTripRecyclerViews.get(position).getDestination());
        TextView rvDistanceTxtView = holder.rvDistanceTxtView;
        rvDistanceTxtView.setText(itemMyTripRecyclerViews.get(position).getDistance());
        TextView rvDurationView = holder.rvDurationView;
        rvDurationView.setText(itemMyTripRecyclerViews.get(position).getDuration());
        TextView rvTotalPriceTxtView = holder.rvTotalPriceTxtView;
        Double tpmDouble = Double.parseDouble(itemMyTripRecyclerViews.get(position).getTotal_price());
        tpmDouble = Functions.roundNumber(tpmDouble, 2);
        rvTotalPriceTxtView.setText(tpmDouble.toString());
        holder.setContext(context);
        setUpSwipeLayout(holder, position);

        rideListSetFavorite(holder, position);

        rowIconAnimation(holder, position);

        rowClickEvents(holder, position);

        setEnableComponents(holder);
    }

    /**
     * This the row click events, will be use in history fragment.
     *
     * @param holder   - is the view holder, for approach view components.
     * @param position - is the item position in RecyclerView.
     */
    private void rowClickEvents(final ViewHolder holder, final int position) {
        holder.imgTripsHistoryIcon.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                listener.onRowTripIconClicked(position);
                setEnableComponents(holder);
            }
        });

        holder.relativeLayoutMainClick.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                listener.onRowHistoryTripClicked(position);
                mainSwipeLayout.close(true);
                setEnableComponents(holder);
            }
        });

        holder.relativeLayoutMainClick.setOnLongClickListener(new View.OnLongClickListener() {
            @Override
            public boolean onLongClick(View view) {
                listener.onRowLongClicked(position);
                setEnableComponents(holder);
                view.performHapticFeedback(HapticFeedbackConstants.LONG_PRESS);
                return true;
            }
        });
    }

    /**
     * Row animation icon, this method is for flip row icon front/back.
     *
     * @param holder   -  is the view holder, for approach view components.
     * @param position - is the item position in RecyclerView.
     */
    private void rowIconAnimation(ViewHolder holder, int position) {
        if (selectedItems.get(position, false)) {
            holder.relativeLayoutMainClick.setBackground(ContextCompat.getDrawable(context, R.drawable.custom_border_list_selected));
            holder.imgTripsHistoryIcon.setVisibility(View.INVISIBLE);
            resetIconYAxis(holder.imgBackSelectedItem);
            holder.imgBackSelectedItem.setVisibility(View.VISIBLE);
            holder.imgBackSelectedItem.setAlpha(1);

            if (currentSelectedIndex == position) {
                Functions.flipView(context, holder.imgBackSelectedItem, holder.imgTripsHistoryIcon, true);
                resetCurrentIndex();
            }
        } else {
            holder.relativeLayoutMainClick.setBackground(ContextCompat.getDrawable(context, R.drawable.custom_border_list));
            holder.imgBackSelectedItem.setVisibility(View.INVISIBLE);
            resetIconYAxis(holder.imgTripsHistoryIcon);
            holder.imgTripsHistoryIcon.setVisibility(View.VISIBLE);
            holder.imgTripsHistoryIcon.setAlpha(1);


            if ((reverseAllAnimations && animationItemsIndex.get(position, false)) || currentSelectedIndex == position) {
                Functions.flipView(context, holder.imgBackSelectedItem, holder.imgTripsHistoryIcon, false);
                resetCurrentIndex();
            }
        }
    }

    /**
     * Sometimes we need to reused the views and the icon is appears as flipped, because of that we need these method.
     * Reset the Y axis to 0.
     *
     * @param view - the view for reset Y axis.
     */
    private void resetIconYAxis(View view) {
        if (view.getRotationY() != 0) {
            view.setRotationY(0);
        }
    }

    /**
     * Reset animation index and clear animation itemsIndex.
     * Array(animationItemsIndex) is used to perform multiple animation at once.
     */
    public void resetAnimationIndex() {
        reverseAllAnimations = false;
        animationItemsIndex.clear();
    }

    /**
     * This method is for select the item on position.
     *
     * @param position - the position of item.
     */
    public void itemSelection(int position) {
        currentSelectedIndex = position;
        if (selectedItems.get(position, false)) {
            selectedItems.delete(position);
            animationItemsIndex.delete(position);
        } else {
            selectedItems.put(position, true);
            animationItemsIndex.put(position, true);
        }
        notifyItemChanged(position);
    }

    /**
     * Set reverse all animations by true.
     * Clear selections and notify data set changed.
     */
    public void clearSelections() {
        reverseAllAnimations = true;
        selectedItems.clear();
        notifyDataSetChanged();
    }

    /**
     * Get the count of selected items.
     *
     * @return count of selected items.
     */
    public int getSelectedItemCount() {
        return selectedItems.size();
    }

    /**
     * Get selected items.
     *
     * @return the integer list of selected items.
     */
    public List<Integer> getSelectedItems() {
        List<Integer> items =
                new ArrayList<>(selectedItems.size());
        for (int i = 0; i < selectedItems.size(); i++) {
            items.add(selectedItems.keyAt(i));
        }
        return items;
    }

    /**
     * Remove dates from itemMyTripRecyclerViews.
     *
     * @param position - remove data on position.
     */
    public void removeData(int position) {
        itemMyTripRecyclerViews.remove(position);
        resetCurrentIndex();
    }

    /**
     * Reset current index of selected item.
     */
    private void resetCurrentIndex() {
        currentSelectedIndex = -1;
    }

    /**
     * Set up swipe layout for delete item.
     *
     * @param holder   - view holder for approach view components.
     * @param position - item position.
     */
    private void setUpSwipeLayout(final ViewHolder holder, final int position) {
        mainSwipeLayout = holder.mainSwipeLayout;
        LinearLayout pullOutLayout = holder.pullOutLayout;
        //set show mode.
        mainSwipeLayout.setShowMode(SwipeLayout.ShowMode.PullOut);
        //add drag edge.
        mainSwipeLayout.addDrag(SwipeLayout.DragEdge.Right, pullOutLayout);
        mainSwipeLayout.addSwipeListener(new SwipeLayout.SwipeListener() {
            @Override
            public void onClose(SwipeLayout layout) {
                //when the SurfaceView totally cover the BottomView.
            }

            @Override
            public void onUpdate(SwipeLayout layout, int leftOffset, int topOffset) {
                //you are swiping.
            }

            @Override
            public void onStartOpen(SwipeLayout layout) {
                // mainSwipeLayout.getDragEdge().name() //left, right
            }

            @Override
            public void onOpen(SwipeLayout layout) {
                //when the BottomView totally show.
            }

            @Override
            public void onStartClose(SwipeLayout layout) {
            }

            @Override
            public void onHandRelease(SwipeLayout layout, float xvel, float yvel) {
                //when user's hand released.
            }
        });
    }

    /**
     * Delete item by id from database.
     *
     * @param position - item position.
     */
    public void removeItem(int position) {
        final ItemMyTripRecyclerView itemForDelete = itemMyTripRecyclerViews.get(position);
        itemMyTripRecyclerViews.remove(position);
        notifyItemRemoved(position);
        notifyDataSetChanged();

        waitTimer = new CountDownTimer(3000, 100) {

            public void onTick(long millisUntilFinished) {
                Log.e("Timer", "time: " + millisUntilFinished);
            }

            public void onFinish() {
                DatabaseHelper databaseHelper = new DatabaseHelper(context);
                databaseHelper.deleteTripById(itemForDelete.getTrip_id());
            }
        }.start();


    }

    public void restoreItem(ItemMyTripRecyclerView item, int position) {
        itemMyTripRecyclerViews.add(position, item);
        // notify item added by position
        notifyItemInserted(position);
    }

    /**
     * Ride all items and set stared if it's needed.
     *
     * @param holder   - view holder for approach view components.
     * @param position - item position.
     */
    private void rideListSetFavorite(ViewHolder holder, int position) {
        if (itemMyTripRecyclerViews.get(position).isFavorite()) {
            holder.imgStaredItem.setImageDrawable(context.getResources().getDrawable(R.drawable.ic_star));
            holder.imgStaredItem.setColorFilter(ContextCompat.getColor(context, R.color.icon_tint_selected));
        } else {
            holder.imgStaredItem.setImageDrawable(context.getResources().getDrawable(R.drawable.ic_star_border));
            holder.imgStaredItem.setColorFilter(ContextCompat.getColor(context, R.color.icon_tint_normal));
        }
    }

    /**
     * Disable all components in row with right swipe for delete, when multichoice is on.
     *
     * @param holder - view holder for approach view components.
     */
    private void setEnableComponents(ViewHolder holder) {
        if (!selectedItems.get(holder.getAdapterPosition(), false)) {
            mainSwipeLayout.setRightSwipeEnabled(true);
            holder.imgStaredItem.setEnabled(true);
        } else {
            mainSwipeLayout.setRightSwipeEnabled(false);
            holder.imgStaredItem.setEnabled(false);

        }
    }

    /**
     * Clear recyclerView if size > 0.
     */
    public void clearRecyclerView() {
        int size = this.itemMyTripRecyclerViews.size();
        if (size > 0) {
            for (int i = 0; i < size; i++) {
                this.itemMyTripRecyclerViews.remove(0);
            }
            this.notifyItemRangeRemoved(0, size);
        }
    }

    /**
     * ViewHolder bind all components from layout here.
     * Set some click lo
     */
    public class ViewHolder extends RecyclerView.ViewHolder {
        private Context context;

        @BindView(R.id.staredItem)
        ImageView imgStaredItem;

        @BindView(R.id.rvOriginTxtView)
        TextView rvOriginTxtView;
        @BindView(R.id.rvDestinationTxtView)
        TextView rvDestinationTxtView;
        @BindView(R.id.rvDistanceTxtView)
        TextView rvDistanceTxtView;
        @BindView(R.id.rvDurationView)
        TextView rvDurationView;
        @BindView(R.id.rvTotalPriceTxtView)
        TextView rvTotalPriceTxtView;

        @BindView(R.id.mainSwipeLayout)
        SwipeLayout mainSwipeLayout;
        @BindView(R.id.pullOutLayout)
        LinearLayout pullOutLayout;
        @BindView(R.id.relativeLayoutMainClick)
        RelativeLayout relativeLayoutMainClick;

        @BindView(R.id.imgBackSelectedItem)
        RelativeLayout imgBackSelectedItem;
        @BindView(R.id.imgTripsHistoryIcon)
        RelativeLayout imgTripsHistoryIcon;


        ViewHolder(View itemView) {
            super(itemView);
            ButterKnife.bind(this, itemView);
        }

        @OnClick(R.id.staredItem)
        void onClickStaredItem(View view) {
            Functions.fadeInAnimation(imgStaredItem, context).setAnimationListener(new Animation.AnimationListener() {
                @Override
                public void onAnimationStart(Animation animation) {
                    if (itemMyTripRecyclerViews.get(getAdapterPosition()).isFavorite()) {
                        setFavorite(false);
                    } else {
                        setFavorite(true);
                    }
                }

                @Override
                public void onAnimationEnd(Animation animation) {
                    //Nothing
                }

                @Override
                public void onAnimationRepeat(Animation animation) {
                    //Nothing
                }
            });
        }

        @OnClick(R.id.deleteItem)
        void onClickDeleteItem(View view) {
            final ItemMyTripRecyclerView restoredItem = itemMyTripRecyclerViews.get(getAdapterPosition());
            final int position = getAdapterPosition();
            mainSwipeLayout.close();
            removeItem(getAdapterPosition());
            Snackbar snackUndo = Snackbar
                    .make(view, "Trip was deleted", Snackbar.LENGTH_LONG)
                    .setAction("UNDO", new View.OnClickListener() {
                        @Override
                        public void onClick(View view) {
                            if (waitTimer != null) {
                                waitTimer.cancel();
                                waitTimer = null;
                            }
                            Snackbar snackRestore = Snackbar.make(view, "Trip is restored!", Snackbar.LENGTH_SHORT);
                            restoreItem(restoredItem, position);
                            snackRestore.show();
                        }
                    });

            snackUndo.show();
        }

        /**
         * @param set
         */
        private void setFavorite(boolean set) {
            if (set) {
                imgStaredItem.setImageDrawable(context.getResources().getDrawable(R.drawable.ic_star));
                imgStaredItem.setColorFilter(ContextCompat.getColor(context, R.color.icon_tint_selected));
                itemMyTripRecyclerViews.get(getAdapterPosition()).setFavorite(true);
                new DatabaseHelper(context).setFavoriteTrip(itemMyTripRecyclerViews.get(getAdapterPosition()).getTrip_id(), 1);
            } else {
                imgStaredItem.setImageDrawable(context.getResources().getDrawable(R.drawable.ic_star_border));
                imgStaredItem.setColorFilter(ContextCompat.getColor(context, R.color.icon_tint_normal));
                itemMyTripRecyclerViews.get(getAdapterPosition()).setFavorite(false);
                new DatabaseHelper(context).setFavoriteTrip(itemMyTripRecyclerViews.get(getAdapterPosition()).getTrip_id(), 0);
            }
        }

        /**
         * @param context
         */
        public void setContext(Context context) {
            this.context = context;
        }
    }

    public interface RecyclerViewHistoryAdapterListener {
        void onRowTripIconClicked(int position);

        void onRowHistoryTripClicked(int position);

        void onRowLongClicked(int position);
    }
}