package com.pavelpetkevich.bcwork.adapters;

import android.content.Context;
import android.content.DialogInterface;
import android.os.Build;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.pavelpetkevich.bcwork.R;
import com.pavelpetkevich.bcwork.googleMapsApi.data.CountryOnWay;

import java.util.ArrayList;

import butterknife.BindView;
import butterknife.ButterKnife;

/**
 * Created by pavelpetkevich on 28/10/2017.
 */

public class RecyclerViewInfoTripAdapter extends RecyclerView.Adapter<RecyclerViewInfoTripAdapter.ViewHolder> {

    private ArrayList<CountryOnWay> itemInfoTripRecyclerViewArrayList;
    private Context context;
    private String consummation;

    public RecyclerViewInfoTripAdapter(ArrayList<CountryOnWay> itemInfoTripRecyclerViewArrayList, Context context, String consummation) {
        this.itemInfoTripRecyclerViewArrayList = itemInfoTripRecyclerViewArrayList;
        this.context = context;
        this.consummation = consummation;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.recycler_view_trip_info_row, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        TextView txtViewCountry = holder.txtViewCountry;
        txtViewCountry.setText(itemInfoTripRecyclerViewArrayList.get(position).getCountry());

        TextView txtViewCountryDistance = holder.txtViewCountryDistance;
        txtViewCountryDistance.setText(itemInfoTripRecyclerViewArrayList.get(position).getDistance());

        TextView txtViewTollCost = holder.txtViewTollCost;
        txtViewTollCost.setText(calculateTollCosts(itemInfoTripRecyclerViewArrayList.get(position).getToll_cost(),
                itemInfoTripRecyclerViewArrayList.get(position).getDistance(), itemInfoTripRecyclerViewArrayList.get(position).getComment()));

        TextView txtViewFuelCost = holder.txtViewFuelCost;

        txtViewFuelCost.setText(calculateFuel(itemInfoTripRecyclerViewArrayList.get(position).getFuel_cost(), itemInfoTripRecyclerViewArrayList.get(position).getDistance(), consummation));

        holder.setComment(itemInfoTripRecyclerViewArrayList.get(position).getComment());
        holder.setCountry(itemInfoTripRecyclerViewArrayList.get(position).getCountry());
        holder.setContext(context);
    }


    @Override
    public int getItemCount() {
        return itemInfoTripRecyclerViewArrayList == null ? 0 : itemInfoTripRecyclerViewArrayList.size();
    }

    static class ViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {
        private Context context;
        private String comment, country;
        @BindView(R.id.txtViewCountry)
        TextView txtViewCountry;
        @BindView(R.id.txtViewCountryDistance)
        TextView txtViewCountryDistance;
        @BindView(R.id.txtViewTollCost)
        TextView txtViewTollCost;
        @BindView(R.id.txtViewFuelCost)
        TextView txtViewFuelCost;

        ViewHolder(View itemView) {
            super(itemView);
            ButterKnife.bind(this, itemView);
            itemView.setOnClickListener(this);
        }

        @Override
        public void onClick(View v) {
            AlertDialog.Builder builder;
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                builder = new AlertDialog.Builder(context, android.R.style.Theme_Material_Dialog_Alert);
            } else {
                builder = new AlertDialog.Builder(context);
            }
            builder.setTitle(country)
                    .setMessage(comment)
                    .setCancelable(false)
                    .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int which) {
                            // continue
                        }
                    })
                    .setIcon(android.R.drawable.ic_dialog_info)
                    .show();
        }

        void setComment(String comment) {
            this.comment = comment;
        }

        void setCountry(String country) {
            this.country = country;
        }

        void setContext(Context context) {
            this.context = context;
        }
    }

    private String calculateTollCosts(String tollCountry, String distance, String comment) {
        ///     getCountriesCostsApi(country.getCountry());
        double toll_cost = 0;
        if (comment.contains("per 100")) {
            toll_cost = Double.parseDouble(distance) / 100 * Double.parseDouble(tollCountry);
        } else if (comment.contains("price is average") || comment.contains("10 days") || comment.contains("weekly") || comment.contains("annual")) {
            toll_cost = Double.parseDouble(tollCountry);
        } else if (comment.contains("Tolls no pay only for one day") || comment.contains("Payment at tollgates") || comment.contains("No pay tolls")) {
            toll_cost = 0;
        }
        return String.valueOf(toll_cost);
    }

    private String calculateFuel(String fuelCost, String distance, String consAverage) {
        if (fuelCost.isEmpty()) fuelCost = "1.5";
        return String.valueOf(Double.parseDouble(distance) / 100 * Double.parseDouble(consAverage) * Double.parseDouble(fuelCost));
    }

    public double getSumFuelCost() {
        double sumFuelCost = 0;
        for (int i = 0; i < itemInfoTripRecyclerViewArrayList.size(); i++) {

            sumFuelCost += Double.valueOf(calculateFuel(itemInfoTripRecyclerViewArrayList.get(i).getFuel_cost(), itemInfoTripRecyclerViewArrayList.get(i).getDistance(), consummation));

        }
        return sumFuelCost;
    }

    public double getSumTollCost() {
        double sumTollCost = 0;
        for (int i = 0; i < itemInfoTripRecyclerViewArrayList.size(); i++) {

            sumTollCost += Double.valueOf(calculateTollCosts(itemInfoTripRecyclerViewArrayList.get(i).getToll_cost(),
                    itemInfoTripRecyclerViewArrayList.get(i).getDistance(), itemInfoTripRecyclerViewArrayList.get(i).getComment()));
        }
        return sumTollCost;
    }
}
