package com.pavelpetkevich.bcwork.adapters;

import android.content.Context;
import android.content.Intent;
import android.support.design.widget.Snackbar;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.pavelpetkevich.bcwork.activity.DetailPlaceActivity;
import com.pavelpetkevich.bcwork.data.ItemPlaceRecyclerView;
import com.pavelpetkevich.bcwork.R;
import com.squareup.picasso.Picasso;

import java.util.ArrayList;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

/**
 * Created by pavelpetkevich on 16/10/2017.
 */

public class RecyclerViewPlaceAdapter extends RecyclerView.Adapter<RecyclerViewPlaceAdapter.ViewHolder>{

    private ArrayList<ItemPlaceRecyclerView> itemPlaceRecyclerViewArrayList;
    private Context context;

    public RecyclerViewPlaceAdapter(ArrayList<ItemPlaceRecyclerView> itemPlaceRecyclerViewArrayList, Context context) {
        this.itemPlaceRecyclerViewArrayList = itemPlaceRecyclerViewArrayList;
        this.context = context;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.fragment_places, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        Picasso.with(context).load(itemPlaceRecyclerViewArrayList.get(position).getImageURL())
                .placeholder(R.drawable.loading).error(R.drawable.something_wrong).into(holder.cardPicture);

        TextView cardDescription = holder.cardDescription;
        cardDescription.setText(itemPlaceRecyclerViewArrayList.get(position).getShortDescription());

        TextView cardTitle = holder.cardTitle;
        cardTitle.setText(itemPlaceRecyclerViewArrayList.get(position).getNameOfPlace());

        holder.setLocation(itemPlaceRecyclerViewArrayList.get(position).getLocation());
        holder.setLongDescription(itemPlaceRecyclerViewArrayList.get(position).getLongDescription());
        holder.setUrlPlaceImg(itemPlaceRecyclerViewArrayList.get(position).getImageURL());

        holder.setContext(context);
    }

    /**
     * Clear recyclerView if size > 0.
     */
    public void clearRecyclerView() {
        int size = this.itemPlaceRecyclerViewArrayList.size();
        if (size > 0) {
            for (int i = 0; i < size; i++) {
                this.itemPlaceRecyclerViewArrayList.remove(0);
            }
            this.notifyItemRangeRemoved(0, size);
        }
    }

    @Override
    public int getItemCount() {
        return itemPlaceRecyclerViewArrayList == null ? 0 : itemPlaceRecyclerViewArrayList.size();
    }


    static class ViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {
        private Context context;
        @BindView(R.id.card_image)
        ImageView cardPicture;
        @BindView(R.id.card_text)
        TextView cardDescription;
        @BindView(R.id.card_title)
        TextView cardTitle;
        private String longDescription, location, urlPlaceImg;

        ViewHolder(View itemView) {
            super(itemView);
            ButterKnife.bind(this, itemView);
            itemView.setOnClickListener(this);
        }

        @Override
        public void onClick(View v) {
            Context context = v.getContext();
            Intent intent = new Intent(context, DetailPlaceActivity.class);
            intent.putExtra("title",cardTitle.getText());
            intent.putExtra("placePicture", urlPlaceImg);
            intent.putExtra("location",location);
            intent.putExtra("longDescription", longDescription);
            context.startActivity(intent);
        }

        @OnClick(R.id.action_button)
        void onClick(){
            Snackbar.make(itemView, "Action is pressed",
                    Snackbar.LENGTH_LONG).show();
        }


        void setContext(Context context) {
            this.context = context;
        }

        void setLongDescription(String longDescription) {
            this.longDescription = longDescription;
        }

        void setLocation(String location) {
            this.location = location;
        }

        void setUrlPlaceImg(String urlPlaceImg) {
            this.urlPlaceImg = urlPlaceImg;
        }
    }
}
