package com.pavelpetkevich.bcwork.auxiliaries;

import android.animation.AnimatorInflater;
import android.animation.AnimatorSet;
import android.content.Context;
import android.location.Address;
import android.location.Geocoder;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.TextInputEditText;
import android.util.Log;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.ImageView;

import com.pavelpetkevich.bcwork.R;
import com.pavelpetkevich.bcwork.data.ItemCarRecyclerView;

import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;
import java.util.Random;

/**
 * Created by pavelpetkevich on 19/11/2017.
 */

public class Functions {

    /**
     * Generate random int for ID.
     *
     * @return integer id.
     */
    public static int randID() {
        return new Random().nextInt(1000000000) + 1;
    }

    /**
     * Animate floating button.
     *
     * @param floatingActionButton - button for rotate animation.
     * @param context              - context for loadAnimation.
     */
    public static void animateFloatingActionButton(FloatingActionButton floatingActionButton, Context context) {
        Animation startRotateAnimation = AnimationUtils.loadAnimation(context, R.anim.rotation_float);
        floatingActionButton.startAnimation(startRotateAnimation);
        floatingActionButton.show();
    }

    /**
     * Rotate animation for ImageView.
     *
     * @param image   - for rotate animation.
     * @param context - for loading animation.
     */
    public static Animation rotateAnimationImageView(final ImageView image, final Context context) {
        Animation startRotateAnimation = AnimationUtils.loadAnimation(context, R.anim.rotation_float);
        image.startAnimation(startRotateAnimation);
        return startRotateAnimation;
    }

    /**
     * Animate image fade in.
     *
     * @param image   - for animate
     * @param context - for loading animation
     * @return
     */
    public static Animation fadeInAnimation(final ImageView image, final Context context) {
        Animation startAnimation = AnimationUtils.loadAnimation(context, R.anim.fade_in);
        image.startAnimation(startAnimation);
        return startAnimation;
    }

    /**
     * Animate image fade out.
     *
     * @param image   - for animate
     * @param context - for loading animation
     * @return
     */
    public static Animation fadeOutAnimation(final ImageView image, final Context context) {
        Animation startAnimation = AnimationUtils.loadAnimation(context, R.anim.fade_out);
        image.startAnimation(startAnimation);
        return startAnimation;
    }

    /**
     * Get date from main string.
     *
     * @param string - main string
     */
    private void getDateFromStringBuilder(String string) {
        if (string != null && !string.isEmpty()) {
            String[] strArray = string.split(";");
            List<String> strList = new ArrayList<>(Arrays.asList(strArray));
            for (int i = 0; i < strList.size(); i++) {
                Log.e("Log", "Some: " + strArray[i]);
            }
        }
    }

    /**
     * Convert car List to Array.
     *
     * @param list - our list of cars
     * @return car array.
     */
    public static String[] convertListToArrayCar(List<ItemCarRecyclerView> list) {
        String[] array = new String[list.size()];
        int index = 0;
        for (ItemCarRecyclerView value : list) {
            array[index] = value.getNameCar() + "\nConsummation: " + value.getConsummation() + " (l/100km)";
            index++;
        }
        return array;
    }

    /**
     * Get address from lat&lng.
     * Using Geocoder.
     *
     * @param lat     - is latitude
     * @param lng     - is longitude
     * @param context - is context.
     * @return
     */
    public static String getAddressFromLatLng(double lat, double lng, Context context) {
        Geocoder geocoder;
        String address = null;
        try {
            geocoder = new Geocoder(context, Locale.getDefault());
            List<Address> addresses = geocoder.getFromLocation(lat, lng, 1); // Here 1 represent max location result to returned, by documents it recommended 1 to 5
            if (addresses.size() != 0)
                if (!addresses.get(0).getFeatureName().equals(addresses.get(0).getAddressLine(0)))
                    addresses.get(0).getAddressLine(0);
            address = addresses.get(0).getFeatureName() + "," + addresses.get(0).getSubAdminArea() + "," + addresses.get(0).getLocality() + "," + addresses.get(0).getCountryName();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return address;
    }

    /**
     * Clear InputEditText.
     *
     * @param textInputEditText - text input
     */
    public static void clearInputEditText(TextInputEditText textInputEditText) {
        textInputEditText.setText("");
        textInputEditText.clearFocus();
    }

    /**
     * Performs flip animation on two views.
     * This method flipping views which we needed.
     *
     * @param context   - the context of application.
     * @param back      - back view which is shown when the showFront is false.
     * @param front     - front view which is shown when the showFront is true.
     * @param showFront - parameter for controlling if it's may be back/front.
     */
    public static void flipView(Context context, final View back, final View front, boolean showFront) {
        AnimatorSet leftIn, rightOut, leftOut, rightIn;
        leftIn = (AnimatorSet) AnimatorInflater.loadAnimator(context, R.animator.card_flip_left_in);
        rightOut = (AnimatorSet) AnimatorInflater.loadAnimator(context, R.animator.card_flip_right_out);
        leftOut = (AnimatorSet) AnimatorInflater.loadAnimator(context, R.animator.card_flip_left_out);
        rightIn = (AnimatorSet) AnimatorInflater.loadAnimator(context, R.animator.card_flip_right_in);

        final AnimatorSet showFrontAnim = new AnimatorSet();
        final AnimatorSet showBackAnim = new AnimatorSet();

        leftIn.setTarget(back);
        rightOut.setTarget(front);
        showFrontAnim.playTogether(leftIn, rightOut);

        leftOut.setTarget(back);
        rightIn.setTarget(front);
        showBackAnim.playTogether(rightIn, leftOut);

        if (showFront) {
            showFrontAnim.start();
        } else {
            showBackAnim.start();
        }
    }


    /**
     * Round number on roundPlace which is needing.
     *
     * @param value       - the value to rounding
     * @param roundPlaces - round the value by count after values after comma.
     * @return the rounded value.
     */
    public static double roundNumber(double value, int roundPlaces) {
        if (roundPlaces < 0) throw new IllegalArgumentException();
        BigDecimal bd = new BigDecimal(value);
        bd = bd.setScale(roundPlaces, RoundingMode.HALF_UP);
        return bd.doubleValue();
    }

}
