package com.pavelpetkevich.bcwork.fragment;

import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.BaseTransientBottomBar;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.design.widget.TextInputEditText;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.widget.NestedScrollView;
import android.support.v7.widget.Toolbar;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.Toast;

import com.pavelpetkevich.bcwork.R;
import com.pavelpetkevich.bcwork.auxiliaries.Functions;
import com.pavelpetkevich.bcwork.data.ItemCarRecyclerView;
import com.pavelpetkevich.bcwork.sql.DatabaseHelper;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

public class AddEditCarFragment extends Fragment {

    @BindView(R.id.carFragImage)
    ImageView carFragImage;
    @BindView(R.id.carFragTxtIntEditTextCarName)
    TextInputEditText carFragTxtIntEditTextCarName;
    @BindView(R.id.carFragTxtInEditTextConsummation)
    TextInputEditText carFragTxtInEditTextConsummation;
    @BindView(R.id.carFragTxtInEditTextLicensePlate)
    TextInputEditText carFragTxtInEditTextLicensePlate;
    @BindView(R.id.carFragTxtInEditTextVIN)
    TextInputEditText carFragTxtInEditTextVIN;

    @BindView(R.id.nestedScrollView)
    NestedScrollView nestedScrollView;
    @BindView(R.id.carFragFloatBtnDone)
    FloatingActionButton carFragFloatBtnDone;


    private DatabaseHelper databaseHelper;
    private ItemCarRecyclerView my_car;

    private Toolbar toolbar;
    private boolean mainBack, edit;
    private int carID;

    /**
     * Empty constructor
     */
    public AddEditCarFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        my_car = new ItemCarRecyclerView();
        databaseHelper = new DatabaseHelper(getActivity());

        toolbar = getActivity().findViewById(R.id.appToolbarMain);
        if (toolbar != null)
            toolbar.setNavigationIcon(R.drawable.ic_arrow_back);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_add_edit_car, container, false);
        ButterKnife.bind(this, view);

        if (getArguments() != null) {
            mainBack = getArguments().getBoolean("mainBack");
            edit = getArguments().getBoolean("edit");
        }

        Functions.animateFloatingActionButton(carFragFloatBtnDone, getContext());

        setViewIfCallForEdit();

        return view;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        setNavigationOnClickListener();
    }

    @Override
    public void onResume() {
        super.onResume();
        onBackMainPressed();
    }

    @Override
    public void onStop() {
        super.onStop();
        //((AppCompatActivity) getActivity()).getSupportActionBar().show();
    }

    @OnClick(R.id.carFragFloatBtnDone)
    public void onClick(View v) {
        if (edit) {
            if (editDataInSqlById()) {
                Snackbar.make(v, R.string.car_edit_success, BaseTransientBottomBar.LENGTH_SHORT).show();
                callFragmentCars();
            } else {
                Snackbar.make(v, R.string.car_edit_failed, BaseTransientBottomBar.LENGTH_SHORT).show();
            }
        } else {
            postDataToSQLite(v);
        }
    }

    /**
     * Post dates to SQLite DB.
     *
     * @param view is for SnackBar
     */
    private void postDataToSQLite(View view) {
        if (edtIsNoEmpty()) {
            setAddCar();
            callFragmentCars();
            Toast.makeText(getActivity(), R.string.car_add_success, Toast.LENGTH_SHORT).show();
            Snackbar.make(view, R.string.car_add_failed, BaseTransientBottomBar.LENGTH_SHORT).show();
        }
    }

    /**
     * Edit existing car in SQLite DB.
     *
     * @return true if all is correct and Edit is successful.
     */
    private boolean editDataInSqlById() {
        Boolean databaseHelperBool = false;
        if (edtIsNoEmpty()) {
            databaseHelperBool = databaseHelper.editCar(carID, carFragTxtIntEditTextCarName.getText().toString(), carFragTxtInEditTextConsummation.getText().toString(),
                    carFragTxtInEditTextLicensePlate.getText().toString(), carFragTxtInEditTextVIN.getText().toString());
        }
        return databaseHelperBool;
    }

    /**
     * After control edtIsNoEmpty(), set and add car to SQLite DB.
     */
    private void setAddCar() {
        my_car.setId(Functions.randID());
        my_car.setNameCar(carFragTxtIntEditTextCarName.getText().toString().trim());
        my_car.setConsummation(Double.parseDouble(carFragTxtInEditTextConsummation.getText().toString().trim()));
        my_car.setLicensePlate(carFragTxtInEditTextLicensePlate.getText().toString().trim());
        my_car.setVin(carFragTxtInEditTextVIN.getText().toString().trim());
        databaseHelper.addCar(my_car);
    }

    /**
     * Control if EditTexts is no empty
     *
     * @return true if all EditTexts is no empty.
     */
    private boolean edtIsNoEmpty() {
        return (!carFragTxtIntEditTextCarName.getText().toString().isEmpty()
                && !carFragTxtInEditTextConsummation.getText().toString().isEmpty()
                && !carFragTxtInEditTextLicensePlate.getText().toString().isEmpty()
                && !carFragTxtInEditTextVIN.getText().toString().isEmpty());
    }

    /**
     * Call MyCarsFragment
     * control if its calling for Edit change animation.
     */
    private void callFragmentCars() {
        FragmentManager fragmentManager = getActivity().getSupportFragmentManager();
        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
        MyCarsFragment myCarsFragment = new MyCarsFragment();
        if (edit) {
            fragmentTransaction.setCustomAnimations(R.anim.enter, R.anim.exit);
        } else {
            fragmentTransaction.setCustomAnimations(R.anim.slide_up, R.anim.slide_down);
        }
        fragmentTransaction.replace(R.id.incl_fragment, myCarsFragment);
        carFragFloatBtnDone.hide();
        fragmentTransaction.commit();
    }

    /**
     * This method for back pressed by main button of phone.
     */
    private void onBackMainPressed() {
        if (getView() == null) {
            return;
        }
        getView().setFocusableInTouchMode(true);
        getView().requestFocus();
        getView().setOnKeyListener(new View.OnKeyListener() {
            @Override
            public boolean onKey(View v, int keyCode, KeyEvent event) {
                if (event.getAction() == KeyEvent.ACTION_UP && keyCode == KeyEvent.KEYCODE_BACK) {
                    carFragFloatBtnDone.hide();
                    if (mainBack) {
                        getActivity().onBackPressed();
                        getActivity().overridePendingTransition(R.anim.enter, R.anim.exit);
                    } else {
                        callFragmentCars();
                    }
                    return true;
                }
                return false;
            }
        });
    }

    /**
     * Set onClickListener on navigation icon back
     */
    private void setNavigationOnClickListener() {
        if (toolbar != null)
            toolbar.setNavigationOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (mainBack) {
                        getActivity().onBackPressed();
                        getActivity().overridePendingTransition(R.anim.enter, R.anim.exit);
                        carFragFloatBtnDone.hide();
                    } else {
                        callFragmentCars();
                    }
                }
            });
    }

    /**
     * This method set view content for edit.
     */
    private void setViewIfCallForEdit() {
        if (edit) {
            carID = getArguments().getInt("_ID");
            carFragTxtIntEditTextCarName.setText(getArguments().getString("carName"));
            carFragTxtInEditTextConsummation.setText(getArguments().getString("carCons"));
            carFragTxtInEditTextLicensePlate.setText(getArguments().getString("carLP"));
            carFragTxtInEditTextVIN.setText(getArguments().getString("carVin"));
            carFragFloatBtnDone.setImageResource(R.drawable.ic_edit_black_24dp);
        }
    }

}
