package com.pavelpetkevich.bcwork.fragment;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.AppBarLayout;
import android.support.design.widget.BaseTransientBottomBar;
import android.support.design.widget.Snackbar;
import android.support.v4.app.Fragment;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.view.ActionMode;
import android.support.v7.widget.DefaultItemAnimator;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.AccelerateInterpolator;
import android.view.animation.DecelerateInterpolator;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.pavelpetkevich.bcwork.R;
import com.pavelpetkevich.bcwork.activity.MainActivity;
import com.pavelpetkevich.bcwork.activity.MapsActivity;
import com.pavelpetkevich.bcwork.adapters.RecyclerViewHistoryAdapter;
import com.pavelpetkevich.bcwork.data.ItemMyTripRecyclerView;

import com.pavelpetkevich.bcwork.sql.DatabaseHelper;
import com.pavelpetkevich.bcwork.dialogs.ShowDialog;
import com.pavelpetkevich.bcwork.service.ControlServices;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

public class HistoryFragment extends Fragment implements SwipeRefreshLayout.OnRefreshListener, RecyclerViewHistoryAdapter.RecyclerViewHistoryAdapterListener {

    @BindView(R.id.my_recycler_view)
    RecyclerView recyclerView;
    @BindView(R.id.imgSomeWrong)
    ImageView imgSomeWrong;
    @BindView(R.id.tv_error_title_primary)
    TextView tv_error_title_primary;
    @BindView(R.id.tv_error_message_secondary)
    TextView tv_error_message_secondary;
    @BindView(R.id.swipe_refresh_layout)
    SwipeRefreshLayout swipeRefreshLayout;

    private boolean showFavoriteOnly = false;
    private List<ItemMyTripRecyclerView> itemList = new ArrayList<>();
    private com.pavelpetkevich.bcwork.sql.DatabaseHelper databaseHelper;
    private RecyclerViewHistoryAdapter adapter;

    private ActionModeCallback actionModeCallback;
    private ActionMode actionMode;

    private AppBarLayout appBarLayout;
    private Toolbar toolbar;

    private Activity mActivity;


    /**
     * Empty public constructor
     */
    public HistoryFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        databaseHelper = new DatabaseHelper(mActivity);
        actionModeCallback = new ActionModeCallback();
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_history, container, false);
        ButterKnife.bind(this, view);

        if (getArguments() != null)
            showFavoriteOnly = getArguments().getBoolean("historyFragment");

        return view;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        setUpRecyclerView();

        appBarLayout = getActivity().findViewById(R.id.appBarMain);
        toolbar = getActivity().findViewById(R.id.appToolbarMain);
        ((AppCompatActivity) getActivity()).getSupportActionBar();

        setSwipeRefreshLayout(true);
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        mActivity = context instanceof Activity ? (Activity) context : null;
    }

    @Override
    public void onResume() {
        super.onResume();
        getDataFromSQLite(false);
    }

    @Override
    public void onPause() {
        super.onPause();
        allInvisible();
        setSwipeRefreshLayout(false);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        ShowDialog.showProgressDialog(false, getString(R.string.progress_dialog_history_text), mActivity);
    }

    @Override
    public void onRefresh() {
        adapter.clearRecyclerView();
        new Handler().postDelayed(new Runnable() {
            @Override
            public void run() {
                getDataFromSQLite(true);
                setSwipeRefreshLayout(false);
            }
        }, 2000);
    }

    /**
     * Here's the onClick listener on image of error.
     * After click on that control if user have all services which we need switch on, if it's not enabled made user to make it.
     *
     * @param view - is only view which we used in Snackbar
     */
    @OnClick(R.id.imgSomeWrong)
    public void onClickImgSomeWrong(View view) {
        if (showFavoriteOnly) {
            Intent intent = new Intent(getActivity(), MainActivity.class);
            Bundle bundle = new Bundle();
            bundle.putBoolean("history", true);
            intent.putExtra("bundle", bundle);
            getActivity().overridePendingTransition(R.anim.enter, R.anim.exit);
            startActivity(intent);
        } else {
            if (ControlServices.controlNetworkStateService(mActivity) && ControlServices.controlGpsProviderServiceIsAvailable(mActivity)) {
                callMakeTripActivity();
            } else if (!ControlServices.controlGpsProviderServiceIsAvailable(mActivity)) {
                Snackbar.make(view, "Control your GPS Provider, if it's switch on.", BaseTransientBottomBar.LENGTH_LONG).show();
                ShowDialog.showLocationServicesDialog(mActivity);
            } else if (!ControlServices.controlNetworkStateService(mActivity)) {
                Snackbar.make(view, "Control your internet connection, if it's switch on.", BaseTransientBottomBar.LENGTH_LONG).show();
                ShowDialog.showWirelessSettingsDialog(mActivity);
            }
        }
    }

    /**
     * Here's the method for get dates from SQLite(History table).
     * Using here AsyncTask for reload and get dates by calling method's from DatabaseHelper.
     * Thread will sleep for 2 seconds, after that in onPostExecute try to getAllTrips from DB.
     * If the list is empty show error message else show list of history.
     */
    @SuppressLint("StaticFieldLeak")
    private void getDataFromSQLite(final boolean isSwipe) {
        // AsyncTask is used that SQLite operation not blocks the UI Thread.
        new AsyncTask<Void, Void, Void>() {
            @Override
            protected void onPreExecute() {
                super.onPreExecute();
                if (showFavoriteOnly) {
                    if (!isSwipe)
                        ShowDialog.showProgressDialog(true, getString(R.string.progress_dialog_important_text), mActivity);
                } else {
                    if (!isSwipe)
                        ShowDialog.showProgressDialog(true, getString(R.string.progress_dialog_history_text), mActivity);
                }
            }

            @Override
            protected Void doInBackground(Void... params) {
                if (!isSwipe) {
                    try {
                        Thread.sleep(2000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
                return null;
            }

            @Override
            protected void onPostExecute(Void aVoid) {
                super.onPostExecute(aVoid);
                if (getActivity() != null) {
                    itemList.clear();
                    itemList.addAll(databaseHelper.getAllTrips(showFavoriteOnly));
                    adapter.sortFavorite();
                    adapter.notifyDataSetChanged();

                    if (itemList.isEmpty() && showFavoriteOnly) {
                        showErrorMessage(R.drawable.ic_star, getString(R.string.empty_important_trips),
                                getString(R.string.empty_important_trips_msg));
                    } else if (itemList.isEmpty()) {
                        showErrorMessage(R.drawable.ic_no_trips_created, getString(R.string.empty_trips),
                                getString(R.string.empty_trips_msg));
                    } else {
                        showHistory();
                    }
                }

            }
        }.execute();
    }

    /**
     * This method show's Error Message and un shown RecyclerView when the list is empty.
     */
    private void showErrorMessage(int errorImg, String title, String msgError) {
        recyclerView.setVisibility(View.INVISIBLE);

        tv_error_title_primary.setVisibility(View.VISIBLE);
        tv_error_message_secondary.setVisibility(View.VISIBLE);
        imgSomeWrong.setVisibility(View.VISIBLE);

        imgSomeWrong.setImageResource(errorImg);
        tv_error_title_primary.setText(title);
        tv_error_message_secondary.setText(msgError);

        ShowDialog.showProgressDialog(false, getString(R.string.progress_dialog_history_text), mActivity);
    }

    /**
     * This method show's history and un shown error views when the list is no empty.
     */
    private void showHistory() {
        tv_error_title_primary.setVisibility(View.INVISIBLE);
        tv_error_message_secondary.setVisibility(View.INVISIBLE);
        imgSomeWrong.setVisibility(View.INVISIBLE);

        recyclerView.setVisibility(View.VISIBLE);
        ShowDialog.showProgressDialog(false, getString(R.string.progress_dialog_history_text), mActivity);
    }

    /**
     * Invisible all views.
     * Using in onPause, for example
     */
    private void allInvisible() {
        tv_error_title_primary.setVisibility(View.INVISIBLE);
        tv_error_message_secondary.setVisibility(View.INVISIBLE);
        imgSomeWrong.setVisibility(View.INVISIBLE);
        recyclerView.setVisibility(View.INVISIBLE);
    }

    /**
     * Set up recycler view.
     */
    private void setUpRecyclerView() {
        adapter = new RecyclerViewHistoryAdapter((ArrayList<ItemMyTripRecyclerView>) itemList, mActivity, this);
        recyclerView.setHasFixedSize(true);
        recyclerView.setLayoutManager(new LinearLayoutManager(mActivity));
        recyclerView.setItemAnimator(new DefaultItemAnimator());
        recyclerView.setAdapter(adapter);
    }

    /**
     * Set up swipe refresh layout.
     *
     * @param set - this boolean param is for control if the swipeRefresh may be available or not.
     */
    private void setSwipeRefreshLayout(boolean set) {
        if (set) {
            swipeRefreshLayout.setOnRefreshListener(this);
            swipeRefreshLayout.setColorSchemeResources(R.color.swipeRefreshAccent, R.color.swipeRefreshAccent2, R.color.swipeRefreshAccent3, R.color.swipeRefreshAccent4);
        } else {
            swipeRefreshLayout.setRefreshing(false);
            swipeRefreshLayout.destroyDrawingCache();
            swipeRefreshLayout.clearAnimation();
        }
    }

    /**
     * Call MapsActivity for create some trip.
     */
    private void callMakeTripActivity() {
        Intent intent = new Intent(mActivity, MapsActivity.class);
        assert mActivity != null;
        mActivity.overridePendingTransition(R.anim.enter, R.anim.exit);
        startActivity(intent);
    }

//Here is starting the part of multiple choice item.

    /**
     * This class is for callback action on create menu of multiple choice.
     * Here we have main methods, like:
     * 1) onCreateActionMode - here is set up our menuInflater, hide up our appBarLayout and disable swipe refresh;
     * 2) onPrepareActionMode - here is can control if our actionMode is on prepare;
     * 3) onActionItemClicked - here is have listeners for clicks on items menu;
     * 4) onDestroyActionMode - here is set visible appBarLayout, clearRecyclerView all selections, enable swipe refresh.
     */
    private class ActionModeCallback implements ActionMode.Callback {

        @Override
        public boolean onCreateActionMode(ActionMode mode, Menu menu) {
            mode.getMenuInflater().inflate(R.menu.action_mode_menu, menu);
            swipeRefreshLayout.setEnabled(false);
            appBarLayout.animate().translationY(-appBarLayout.getBottom()).setInterpolator(new AccelerateInterpolator()).start();
            toolbar.animate().translationY(-toolbar.getBottom()).setInterpolator(new AccelerateInterpolator()).start();
            appBarLayout.setVisibility(View.GONE);
            return true;
        }

        @Override
        public boolean onPrepareActionMode(ActionMode mode, Menu menu) {
            return false;
        }

        @Override
        public boolean onActionItemClicked(ActionMode mode, MenuItem item) {
            switch (item.getItemId()) {
                case R.id.action_delete:
                    // delete all the selected messages
                    deleteSelectedTrip();
                    mode.finish();
                    return true;
                case R.id.action_select_all:
                    selectUnselectedAll(item);
                    return true;

                default:
                    return false;
            }
        }

        @Override
        public void onDestroyActionMode(ActionMode mode) {
            appBarLayout.setVisibility(View.VISIBLE);
            toolbar.animate().translationY(0).setInterpolator(new DecelerateInterpolator()).start();
            appBarLayout.animate().translationY(0).setInterpolator(new DecelerateInterpolator()).start();
            adapter.clearSelections();
            swipeRefreshLayout.setEnabled(true);

            actionMode = null;
            recyclerView.post(new Runnable() {
                @Override
                public void run() {
                    adapter.resetAnimationIndex();
                    adapter.notifyDataSetChanged();
                }
            });
        }

    }

    @Override
    public void onRowTripIconClicked(int position) {
        enableActionMode(position);
        controlIsAllSelect(actionMode.getMenu().findItem(R.id.action_select_all));
    }

    @Override
    public void onRowHistoryTripClicked(int position) {
        if (adapter.getSelectedItemCount() > 0) {
            enableActionMode(position);
            if (actionMode != null)
                controlIsAllSelect(actionMode.getMenu().findItem(R.id.action_select_all));
        } else {
            Toast.makeText(mActivity, "Work on that.", Toast.LENGTH_SHORT).show();
        }

    }

    @Override
    public void onRowLongClicked(int position) {
        enableActionMode(position);
    }

    /**
     * Here is control if actionMode isn't null, else startSupportActionMode(callback).
     * If actionMode isn't null call itemSelection and push there item position.
     *
     * @param position - position of item in RecyclerView.
     */
    private void enableActionMode(int position) {
        if (actionMode == null) {
            assert getActivity() != null;
            actionMode = ((AppCompatActivity) getActivity()).startSupportActionMode(actionModeCallback);
        }
        itemSelection(position);
    }

    /**
     * Here is the method which make item selected/unselected.
     * If the count of selected item is 0 finish mode.
     * Else set title of count selected items and
     * invalidate() - invalidate the action mode and refresh menu content.
     *
     * @param position
     */
    private void itemSelection(int position) {
        adapter.itemSelection(position);
        int count = adapter.getSelectedItemCount();

        if (count == 0) {
            actionMode.finish();
        } else {
            actionMode.setTitle(String.valueOf(count));
            actionMode.invalidate();
        }
    }

    /**
     * Deleting all selected messages from recycler view and db
     */
    private void deleteSelectedTrip() {
        adapter.resetAnimationIndex();
        List<Integer> selectedItemPositions =
                adapter.getSelectedItems();
        for (int i = selectedItemPositions.size() - 1; i >= 0; i--) {
            adapter.removeData(selectedItemPositions.get(i));
            //adapter.removeItem(selectedItemPositions.get(i));
        }
        adapter.notifyDataSetChanged();
    }

    /**
     * Select or Unselected:
     * 1) First of all control if need to set title of item Select/Unselected
     * 2) If it's true set title Unselected and unselecting all items and finish actionMode.
     * 3) Else select all.
     *
     * @param menuItem - this menu item Select/Unselected.
     */
    private void selectUnselectedAll(MenuItem menuItem) {
        if (controlIsAllSelect(menuItem)) {
            adapter.clearSelections();
            actionMode.finish();
        } else {
            adapter.clearSelections();
            for (int i = 0; i < adapter.getItemCount(); i++) {
                adapter.itemSelection(i);
            }
            actionMode.setTitle(String.valueOf(adapter.getSelectedItemCount()));
        }
        controlIsAllSelect(menuItem);
    }

    /**
     * Control if need to set title of item Select/Unselected
     *
     * @param menuItem - this menu item Select/Unselected.
     * @return true if it's Unselected else false.
     */
    private boolean controlIsAllSelect(MenuItem menuItem) {
        if (adapter.getSelectedItemCount() == adapter.getItemCount()) {
            menuItem.setTitle(R.string.unselect_all);
            return true;
        } else {
            menuItem.setTitle(R.string.select_all);
            return false;
        }
    }
}
