package com.pavelpetkevich.bcwork.fragment;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.BaseTransientBottomBar;
import android.support.design.widget.Snackbar;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import com.pavelpetkevich.bcwork.R;
import com.pavelpetkevich.bcwork.activity.FavoriteTripActivity;
import com.pavelpetkevich.bcwork.activity.MapsActivity;
import com.pavelpetkevich.bcwork.activity.MyCarsActivity;
import com.pavelpetkevich.bcwork.dialogs.ShowDialog;
import com.pavelpetkevich.bcwork.service.ControlServices;

import butterknife.ButterKnife;
import butterknife.OnClick;

public class HomeFragment extends Fragment {
    private final static int REQUEST_CODE_PERMISSION_LOCATION = 1;

    //TODO decide if it's needed to make advertising.
    //@BindView(R.id.adViewBottom)
    //AdView mAdViewBottom;

    protected Context context;

    public HomeFragment() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_home, container, false);
        ButterKnife.bind(this, view);
        return view;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        //AdRequest adRequest = new AdRequest.Builder()
        //        .addTestDevice("33BE2250B43518CCDA7DE426D04EE232")
        //        .build();
        //  mAdViewBottom.loadAd(adRequest);
    }

    @OnClick({R.id.relMakeTrip})
    public void onClickMakeTrip(View view) {
        if (ControlServices.controlNetworkStateService(getActivity()) && ControlServices.controlGpsProviderServiceIsAvailable(getActivity())) {
            callMakeTripActivity();
        } else if (!ControlServices.controlGpsProviderServiceIsAvailable(getActivity())) {
            Snackbar.make(view, R.string.warning_gps_provider, BaseTransientBottomBar.LENGTH_LONG).show();
            ShowDialog.showLocationServicesDialog(getActivity());
        } else if (!ControlServices.controlNetworkStateService(getActivity())) {
            Snackbar.make(view, R.string.warning_internet_connection, BaseTransientBottomBar.LENGTH_LONG).show();
            ShowDialog.showWirelessSettingsDialog(getActivity());
        }
    }

    @OnClick({R.id.relMyCars})
    public void onClickMyCars(View view) {
        if (getContext() != null)
            getContext().startActivity(new Intent(getActivity(), MyCarsActivity.class));
        ((Activity) getContext()).overridePendingTransition(R.anim.enter, R.anim.exit);
    }

    @OnClick({R.id.relMyTrips})
    public void onClickMyTrips(View view) {
        if (getContext() != null)
            getContext().startActivity(new Intent(getActivity(), FavoriteTripActivity.class));
        ((Activity) getContext()).overridePendingTransition(R.anim.enter, R.anim.exit);
    }

    @Override
    public void onPause() {
        super.onPause();
    }

    @Override
    public void onResume() {
        super.onResume();
        mainControl(getView());
    }

    /**
     * This is the RequestPermissionsResult which is call automatically.
     *
     * @param requestCode  - is request code for permission
     * @param permissions  - our's array of permissions
     * @param grantResults - is the int array of grantResults(if request is cancelled, the result arrays are empty)
     */
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        switch (requestCode) {
            case REQUEST_CODE_PERMISSION_LOCATION: {
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    Toast.makeText(getActivity(), R.string.permission_confirm, Toast.LENGTH_LONG).show();
                } else {
                    Toast.makeText(getActivity(), R.string.permission_decline, Toast.LENGTH_LONG).show();
                }
            }
        }
    }

    /**
     * fineCoarseLocationPermission() control if permission is needed
     */
    private void fineCoarseLocationPermission() {
        if (ActivityCompat.checkSelfPermission(getActivity(), Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED &&
                ActivityCompat.checkSelfPermission(getActivity(), Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(getActivity(), new String[]{Manifest.permission.ACCESS_COARSE_LOCATION}, 1);
        } else {
            Toast.makeText(getActivity(), R.string.permission_already_given, Toast.LENGTH_LONG).show();
        }
    }

    /**
     * mainControl() - this method is the final control all services and permissions.
     * After control method decide if the user can direct to MapsActivity for create some trip.
     * Direct to MapsActivity is made by method - callMakeTripActivity() .
     *
     * @param view - for snackBar
     */
    private void mainControl(View view) {
        if (ControlServices.controlNetworkStateService(getActivity())) {
            if (ControlServices.controlGpsProviderServiceIsAvailable(getActivity())) {
                fineCoarseLocationPermission();
            } else if (!ControlServices.controlGpsProviderServiceIsAvailable(getActivity())) {
                Snackbar.make(view, R.string.warning_gps_provider, BaseTransientBottomBar.LENGTH_LONG).show();
                ShowDialog.showLocationServicesDialog(getActivity());
            }
        } else if (!ControlServices.controlNetworkStateService(getActivity())) {
            Snackbar.make(view, R.string.warning_internet_connection, BaseTransientBottomBar.LENGTH_LONG).show();
            ShowDialog.showWirelessSettingsDialog(getActivity());
        }
    }

    /**
     * Call MapsActivity for create some trip.
     */
    private void callMakeTripActivity() {
        Intent intent = new Intent(getActivity(), MapsActivity.class);
        assert getContext() != null;
        ((Activity) getContext()).overridePendingTransition(R.anim.enter, R.anim.exit);
        startActivity(intent);
    }
}
