package com.pavelpetkevich.bcwork.fragment;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.DefaultItemAnimator;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.support.v7.widget.helper.ItemTouchHelper;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.pavelpetkevich.bcwork.R;
import com.pavelpetkevich.bcwork.adapters.RecyclerViewCarAdapter;
import com.pavelpetkevich.bcwork.auxiliaries.Functions;
import com.pavelpetkevich.bcwork.data.ItemCarRecyclerView;
import com.pavelpetkevich.bcwork.sql.DatabaseHelper;

import java.util.ArrayList;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

public class MyCarsFragment extends Fragment {

    @BindView(R.id.my_recycler_view)
    RecyclerView recyclerView;
    @BindView(R.id.myCarFragFloatBtnAdd)
    FloatingActionButton myCarFragFloatBtnAdd;

    @BindView(R.id.swipe_refresh_layout)
    SwipeRefreshLayout swipeRefreshLayout;

    private ArrayList<ItemCarRecyclerView> itemList;
    private RecyclerViewCarAdapter adapter;
    private DatabaseHelper databaseHelper;
    private Toolbar toolbar;
    /**
     * Empty constructor
     */
    public MyCarsFragment() {
        // Required empty public constructor
    }


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        databaseHelper = new DatabaseHelper(getActivity());

        itemList = new ArrayList<>();
        adapter = new RecyclerViewCarAdapter(itemList, getActivity());

        toolbar = getActivity().findViewById(R.id.appToolbarMain);
        if (toolbar != null)
            toolbar.setNavigationIcon(R.drawable.ic_arrow_back); toolbar.setTitle(R.string.my_cars_frag_title);
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_my_cars, container, false);
        ButterKnife.bind(this, view);
        swipeRefreshLayout.setEnabled(false);
        return view;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        setRecyclerView();

        Functions.animateFloatingActionButton(myCarFragFloatBtnAdd, getContext());

        setNavigationOnClickListener();

        getDataFromSQLite();

        ItemTouchHelper itemTouchHelper = new ItemTouchHelper(createHelperCallback());
        itemTouchHelper.attachToRecyclerView(recyclerView);
    }

    @OnClick(R.id.myCarFragFloatBtnAdd)
    public void onClick(View v) {
        callFragmentAddCars();
    }

    @Override
    public void onResume() {
        super.onResume();
        onBackMainPressed();
    }

    /**
     * This callback is for item swipe/drag control
     *
     * @return True if item is moved.
     */
    private ItemTouchHelper.Callback createHelperCallback() {
        final ItemTouchHelper.SimpleCallback simpleItemTouchCallback =
                new ItemTouchHelper.SimpleCallback(ItemTouchHelper.UP | ItemTouchHelper.DOWN,
                        ItemTouchHelper.LEFT | ItemTouchHelper.RIGHT) {

                    @Override
                    public boolean onMove(RecyclerView recyclerView, RecyclerView.ViewHolder viewHolder,
                                          RecyclerView.ViewHolder target) {
                        moveItem(viewHolder.getAdapterPosition(), target.getAdapterPosition());
                        return true;
                    }

                    @Override
                    public void onSwiped(final RecyclerView.ViewHolder viewHolder, int swipeDir) {
                        deleteItem(viewHolder.getAdapterPosition());
                    }
                };
        return simpleItemTouchCallback;
    }

    /**
     * MoveItem is helpful method which using the method above for moving some items in recyclerView
     *
     * @param oldPos - old item position
     * @param newPos - new item position
     */
    private void moveItem(int oldPos, int newPos) {
        ItemCarRecyclerView item = itemList.get(oldPos);
        itemList.remove(oldPos);
        itemList.add(newPos, item);
        adapter.notifyItemMoved(oldPos, newPos);
    }

    /**
     * DeleteItem is helpful method which using the method above for deleting some item from DB&RecyclerView
     *
     * @param position - item position in RecyclerView for deleting
     */
    private void deleteItem(final int position) {
        databaseHelper.deleteCarById(itemList.get(position).getId());
        itemList.remove(position);
        adapter.notifyItemRemoved(position);
        if (itemList.isEmpty()) {
            callFragmentAddCars();
        }
    }

    /**
     * Get dates from SQLite database by AsyncTask(use the method from DatabaseHelper).
     */
    @SuppressLint("StaticFieldLeak")
    private void getDataFromSQLite() {
        // AsyncTask is used that SQLite operation not blocks the UI Thread.
        new AsyncTask<Void, Void, Void>() {
            @Override
            protected Void doInBackground(Void... params) {
                itemList.clear();
                itemList.addAll(databaseHelper.getAllCars());
                return null;
            }

            @Override
            protected void onPostExecute(Void aVoid) {
                super.onPostExecute(aVoid);
                adapter.notifyDataSetChanged();
            }
        }.execute();
    }

    /**
     * Call fragment with Add||Edit functionality.
     * Using here some bundles for set where user can be back from AddEditCarFragment.
     * If list is empty user can't see the fragment with RecyclerView which hasn't some items!
     * This is the main cause for using bundles (mainBack) - if it's true return to MAIN_WINDOW else to AddEditCarFragment.
     */
    private void callFragmentAddCars() {
        FragmentManager fragmentManager = getActivity().getSupportFragmentManager();
        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
        AddEditCarFragment addCarFragment = new AddEditCarFragment();
        fragmentTransaction.setCustomAnimations(R.anim.slide_up, R.anim.slide_down);
        myCarFragFloatBtnAdd.hide();
        Bundle bundle = new Bundle();
        if (itemList.isEmpty()) {
            bundle.putBoolean("mainBack", true);
        } else {
            bundle.putBoolean("mainBack", false);
        }
        bundle.putBoolean("edit", false);
        addCarFragment.setArguments(bundle);
        fragmentTransaction.replace(R.id.incl_fragment, addCarFragment);
        fragmentTransaction.commit();
    }

    /**
     * Set onClickListener on navigation icon back
     */
    private void setNavigationOnClickListener() {
        if (toolbar != null)
            toolbar.setNavigationOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    getActivity().onBackPressed();
                    assert getContext() != null;
                    ((Activity) getContext()).overridePendingTransition(R.anim.enter, R.anim.exit);
                    myCarFragFloatBtnAdd.hide();
                }
            });
    }

    /**
     * Set recycler view: Margins, FixedSize, ItemAnimator, Set adapter.
     */
    private void setRecyclerView() {
        recyclerView.setHasFixedSize(true);
        recyclerView.setLayoutManager(new LinearLayoutManager(getActivity()));
        recyclerView.setItemAnimator(new DefaultItemAnimator());
        recyclerView.setAdapter(adapter);
    }

    /**
     * This method for back pressed by main button of phone.
     */
    private void onBackMainPressed() {
        if (getView() == null) {
            return;
        }
        getView().setFocusableInTouchMode(true);
        getView().requestFocus();
        getView().setOnKeyListener(new View.OnKeyListener() {
            @Override
            public boolean onKey(View v, int keyCode, KeyEvent event) {
                if (event.getAction() == KeyEvent.ACTION_UP && keyCode == KeyEvent.KEYCODE_BACK) {
                    getActivity().onBackPressed();
                    getActivity().overridePendingTransition(R.anim.enter, R.anim.exit);
                    return true;
                }
                return false;
            }
        });
    }
}
