package com.pavelpetkevich.bcwork.sql;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import com.pavelpetkevich.bcwork.data.ItemCarRecyclerView;
import com.pavelpetkevich.bcwork.data.ItemMyTripRecyclerView;

import java.util.ArrayList;
import java.util.List;


public class DatabaseHelper extends SQLiteOpenHelper {
    // Database Version
    private static final int DATABASE_VERSION = 5;
    // Database Name
    private static final String DATABASE_NAME = "TrapDB.db";

    private final static String DROP_CAR_TABLE = "DROP TABLE IF EXISTS " + CarContract.CarEntry.TABLE_NAME;
    private final static String DROP_TRIP_TABLE = "DROP TABLE IF EXISTS " + TripContract.TripEntry.TABLE_NAME;


    private DatabaseHelper DBHelper;
    private SQLiteDatabase db;

    public DatabaseHelper(Context context) {
        super(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    @Override
    public void onCreate(SQLiteDatabase db) {
        final String SQL_CREATE_CAR_TABLE = "CREATE TABLE IF NOT EXISTS " + CarContract.CarEntry.TABLE_NAME + " (" +
                CarContract.CarEntry._ID + " INTEGER NOT NULL," +
                CarContract.CarEntry.COLUMN_CAR_NAME + " TEXT NOT NULL, " +
                CarContract.CarEntry.COLUMN_CAR_CONSUMMATION + " REAL NOT NULL, " +
                CarContract.CarEntry.COLUMN_CAR_LICENSE_PLATE + " TEXT NOT NULL, " +
                CarContract.CarEntry.COLUMN_CAR_VIN + " TEXT NOT NULL " +
                "); ";
        db.execSQL(SQL_CREATE_CAR_TABLE);

        final String SQL_CREATE_TRIP_TABLE = "CREATE TABLE IF NOT EXISTS " + TripContract.TripEntry.TABLE_NAME + " (" +
                TripContract.TripEntry._ID + " INTEGER NOT NULL, " +
                TripContract.TripEntry.COLUMN_ORIGIN + " TEXT NOT NULL, " +
                TripContract.TripEntry.COLUMN_DESTINATION + " TEXT NOT NULL, " +
                TripContract.TripEntry.COLUMN_DISTANCE + " TEXT NOT NULL, " +
                TripContract.TripEntry.COLUMN_DURATION + " TEXT NOT NULL, " +
                TripContract.TripEntry.COLUMN_TOTAL_PRICE + " TEXT NOT NULL, " +
                TripContract.TripEntry.COLUMN_FAVORITE + " INTEGER NOT NULL, " +
                TripContract.TripEntry.COLUMN_FK_MY_CAR + " INTEGER NOT NULL " +
                "); ";
        db.execSQL(SQL_CREATE_TRIP_TABLE);
    }

    //---opens the database---
    public DatabaseHelper open() throws SQLException {
        db = DBHelper.getWritableDatabase();
        return this;
    }

    //---closes the database---
    public void close() {
        DBHelper.close();
    }

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        //Drop Car Table if exist
        db.execSQL(DROP_CAR_TABLE);
        //Drop Trip Table if exist
        db.execSQL(DROP_TRIP_TABLE);
        // Create tables again
        onCreate(db);
    }

    //CAR_TABLE_WORK_SPACE

    public void deleteCarById(int id) {
        SQLiteDatabase db = this.getWritableDatabase();
        db.execSQL("DELETE FROM " + CarContract.CarEntry.TABLE_NAME + " WHERE " + CarContract.CarEntry._ID + " " + " LIKE '%" + id + "%'"); //delete all rows in a table
        db.close();
    }

    public void addCar(ItemCarRecyclerView itemCarRecyclerView) {
        SQLiteDatabase db = this.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(CarContract.CarEntry._ID, itemCarRecyclerView.getId());
        values.put(CarContract.CarEntry.COLUMN_CAR_NAME, itemCarRecyclerView.getNameCar());
        values.put(CarContract.CarEntry.COLUMN_CAR_CONSUMMATION, itemCarRecyclerView.getConsummation());
        values.put(CarContract.CarEntry.COLUMN_CAR_LICENSE_PLATE, itemCarRecyclerView.getLicensePlate());
        values.put(CarContract.CarEntry.COLUMN_CAR_VIN, itemCarRecyclerView.getVin());

        db.insert(CarContract.CarEntry.TABLE_NAME, null, values);
        db.close();
    }

    public boolean editCar(int carId, String carName, String carCons, String carLP, String carVin) {
        SQLiteDatabase db = this.getWritableDatabase();
        String sql = "update " + CarContract.CarEntry.TABLE_NAME + " SET " +
                CarContract.CarEntry.COLUMN_CAR_NAME + "= '" + carName + "', " +
                CarContract.CarEntry.COLUMN_CAR_CONSUMMATION + "= '" + Double.parseDouble(carCons) + "', " +
                CarContract.CarEntry.COLUMN_CAR_LICENSE_PLATE + "= '" + carLP + "', " +
                CarContract.CarEntry.COLUMN_CAR_VIN + "= '" + carVin + "' WHERE " + CarContract.CarEntry._ID + " " + " LIKE '%" + carId + "%'";
        try {
            db.execSQL(sql);
            return true;
        } catch (SQLException ex) {
            //TODO Make some exMSG
            return false;
        }

    }

    public List<ItemCarRecyclerView> getAllCars() {
        List<ItemCarRecyclerView> list = new ArrayList<>();

        String[] columns = {
                CarContract.CarEntry._ID,
                CarContract.CarEntry.COLUMN_CAR_NAME,
                CarContract.CarEntry.COLUMN_CAR_CONSUMMATION,
                CarContract.CarEntry.COLUMN_CAR_LICENSE_PLATE,
                CarContract.CarEntry.COLUMN_CAR_VIN
        };

        String sortOrder = CarContract.CarEntry.COLUMN_CAR_NAME + " ASC";

        SQLiteDatabase db1 = this.getReadableDatabase();

        Cursor cursor = db1.query(CarContract.CarEntry.TABLE_NAME, //Table to query
                columns,    //columns to return
                null,        //columns for the WHERE clause
                null,        //The values for the WHERE clause
                null,       //group the rows
                null,       //filter by row groups
                sortOrder); //The sort order

        // Traversing through all rows and adding to list
        if (cursor.moveToFirst()) {
            do {
                ItemCarRecyclerView itemCarRecyclerView = new ItemCarRecyclerView();
                itemCarRecyclerView.setId(cursor.getInt(cursor.getColumnIndex(CarContract.CarEntry._ID)));
                itemCarRecyclerView.setNameCar(cursor.getString(cursor.getColumnIndex(CarContract.CarEntry.COLUMN_CAR_NAME)));
                itemCarRecyclerView.setConsummation(Double.parseDouble(cursor.getString(cursor.getColumnIndex(CarContract.CarEntry.COLUMN_CAR_CONSUMMATION))));
                itemCarRecyclerView.setLicensePlate(cursor.getString(cursor.getColumnIndex(CarContract.CarEntry.COLUMN_CAR_LICENSE_PLATE)));
                itemCarRecyclerView.setVin(cursor.getString(cursor.getColumnIndex(CarContract.CarEntry.COLUMN_CAR_VIN)));
                list.add(itemCarRecyclerView);
            } while (cursor.moveToNext());
        }
        cursor.close();
        db1.close();

        return list;
    }

    //TRIP_TABLE_WORK_SPACE

    public void addTrip(ItemMyTripRecyclerView itemMyTripRecyclerView) {
        SQLiteDatabase db = this.getWritableDatabase();

        ContentValues values = new ContentValues();

        values.put(TripContract.TripEntry._ID, itemMyTripRecyclerView.getTrip_id());
        values.put(TripContract.TripEntry.COLUMN_ORIGIN, itemMyTripRecyclerView.getOrigin());
        values.put(TripContract.TripEntry.COLUMN_DESTINATION, itemMyTripRecyclerView.getDestination());
        values.put(TripContract.TripEntry.COLUMN_DISTANCE, itemMyTripRecyclerView.getDistance());
        values.put(TripContract.TripEntry.COLUMN_DURATION, itemMyTripRecyclerView.getDuration());
        values.put(TripContract.TripEntry.COLUMN_TOTAL_PRICE, itemMyTripRecyclerView.getTotal_price());
        values.put(TripContract.TripEntry.COLUMN_FAVORITE, itemMyTripRecyclerView.isFavorite());
        values.put(TripContract.TripEntry.COLUMN_FK_MY_CAR, itemMyTripRecyclerView.getFk_my_car());

        db.insert(TripContract.TripEntry.TABLE_NAME, null, values);
        db.close();
    }

    public void deleteTripById(int id) {
        SQLiteDatabase db = this.getWritableDatabase();
        db.execSQL("DELETE FROM " + TripContract.TripEntry.TABLE_NAME + " WHERE " + TripContract.TripEntry._ID + " " + " LIKE '%" + id + "%'");
        db.close();
    }

    public boolean setFavoriteTrip(int tripId, int favorite) {
        SQLiteDatabase db = this.getWritableDatabase();
        String sql = "UPDATE " + TripContract.TripEntry.TABLE_NAME + " SET " + TripContract.TripEntry.COLUMN_FAVORITE +
                " = '" + favorite + "' WHERE " + CarContract.CarEntry._ID + " " + " LIKE '%" + tripId + "%'";
        try {
            db.execSQL(sql);
            return true;
        } catch (SQLException ex) {
            //TODO Make some exMSG
            return false;
        }
    }

    public List<ItemMyTripRecyclerView> getAllTrips(boolean showFavoriteOnly) {
        List<ItemMyTripRecyclerView> list = new ArrayList<>();

        String[] columns = {
                TripContract.TripEntry._ID,
                TripContract.TripEntry.COLUMN_ORIGIN,
                TripContract.TripEntry.COLUMN_DESTINATION,
                TripContract.TripEntry.COLUMN_DISTANCE,
                TripContract.TripEntry.COLUMN_DURATION,
                TripContract.TripEntry.COLUMN_TOTAL_PRICE,
                TripContract.TripEntry.COLUMN_FAVORITE,
                TripContract.TripEntry.COLUMN_FK_MY_CAR
        };

        String sortOrder = TripContract.TripEntry._ID + " ASC ";

        SQLiteDatabase db1 = this.getReadableDatabase();

        Cursor cursor;
        if (showFavoriteOnly) {
            cursor = db1.query(TripContract.TripEntry.TABLE_NAME, //Table to query
                    columns,    //columns to return
                    TripContract.TripEntry.COLUMN_FAVORITE,        //columns for the WHERE clause
                    null,        //The values for the WHERE clause
                    null,       //group the rows
                    null,       //filter by row groups
                    sortOrder); //The sort order
        } else {
            cursor = db1.query(TripContract.TripEntry.TABLE_NAME, //Table to query
                    columns,    //columns to return
                    null,        //columns for the WHERE clause
                    null,        //The values for the WHERE clause
                    null,       //group the rows
                    null,       //filter by row groups
                    sortOrder); //The sort order
        }

        // Traversing through all rows and adding to list
        if (cursor.moveToFirst()) {
            do {
                ItemMyTripRecyclerView itemMyTripRecyclerView = new ItemMyTripRecyclerView();

                itemMyTripRecyclerView.setTrip_id(cursor.getInt(cursor.getColumnIndex(TripContract.TripEntry._ID)));
                itemMyTripRecyclerView.setOrigin(cursor.getString(cursor.getColumnIndex(TripContract.TripEntry.COLUMN_ORIGIN)));
                itemMyTripRecyclerView.setDestination(cursor.getString(cursor.getColumnIndex(TripContract.TripEntry.COLUMN_DESTINATION)));
                itemMyTripRecyclerView.setDistance(cursor.getString(cursor.getColumnIndex(TripContract.TripEntry.COLUMN_DISTANCE)));
                itemMyTripRecyclerView.setDuration(cursor.getString(cursor.getColumnIndex(TripContract.TripEntry.COLUMN_DURATION)));
                itemMyTripRecyclerView.setTotal_price(cursor.getString(cursor.getColumnIndex(TripContract.TripEntry.COLUMN_TOTAL_PRICE)));
                itemMyTripRecyclerView.setFavorite(cursor.getInt(cursor.getColumnIndex(TripContract.TripEntry.COLUMN_FAVORITE)) > 0);
                itemMyTripRecyclerView.setFk_my_car(Integer.parseInt(cursor.getString(cursor.getColumnIndex(TripContract.TripEntry.COLUMN_FK_MY_CAR))));

                list.add(itemMyTripRecyclerView);
            } while (cursor.moveToNext());
        }
        cursor.close();
        db1.close();

        return list;
    }


}
