package com.pavelpetkevich.bcwork.dialogs;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.provider.Settings;
import android.view.WindowManager;
import android.widget.Toast;

import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnMyLocationChangeListener;
import com.pavelpetkevich.bcwork.R;
import com.pavelpetkevich.bcwork.fragment.HistoryFragment;

/**
 * Created by pavelpetkevich on 18/11/2017.
 */

public class ShowDialog {
    private static ProgressDialog progressDialog;

    /**
     * This method is for show dialog to user, if Location Services is switch off.
     * If user click on positive button direct user to settings where user can switch on Location Services.
     *
     * @param activity we need the instance of activity for methods which we can call only from activity.
     */
    public static void showLocationServicesDialog(final Activity activity) {
        final AlertDialog.Builder dialog = new AlertDialog.Builder(activity, R.style.AlertDialogCustomStyle);
        dialog.setTitle(R.string.dialog_location_title);
        dialog.setMessage(R.string.dialog_location_msg);
        dialog.setPositiveButton(R.string.dialog_switch_on_button, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface paramDialogInterface, int paramInt) {
                Intent myIntent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                activity.startActivity(myIntent);
            }
        });
        dialog.setNegativeButton(R.string.dialog_cancel_button, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface paramDialogInterface, int paramInt) {
                Toast.makeText(activity, R.string.dialog_location_warning_msg, Toast.LENGTH_LONG).show();
            }
        });
        dialog.setCancelable(false);
        dialog.show();
    }

    /**
     * This method is for show dialog to user, if Wireless Settings is not setup,
     * If user click on positive button direct user to Wireless Settings and ask him for switch on WIFI or Mobile Network.
     *
     * @param activity we need the instance of activity for methods which we can call only from activity.
     */
    public static void showWirelessSettingsDialog(final Activity activity) {
        final AlertDialog.Builder dialog = new AlertDialog.Builder(activity, R.style.AlertDialogCustomStyle);
        dialog.setTitle(R.string.dialog_wireless_title);
        dialog.setMessage(R.string.dialog_wireless_msg);


        dialog.setPositiveButton(R.string.dialog_custom_wireless_wifi_button, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface paramDialogInterface, int paramInt) {
                Intent myIntent = new Intent(Settings.ACTION_WIFI_SETTINGS);
                activity.startActivity(myIntent);
            }
        });
        dialog.setNegativeButton(R.string.dialog_custom_wireless_mobile_button, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                Intent myIntent = new Intent(Settings.ACTION_WIRELESS_SETTINGS);
                activity.startActivity(myIntent);
            }
        });
        dialog.setNeutralButton(R.string.dialog_cancel_button, new DialogInterface.OnClickListener() {

            @Override
            public void onClick(DialogInterface paramDialogInterface, int paramInt) {
                Toast.makeText(activity, R.string.dialog_wireless_warning_msg, Toast.LENGTH_LONG).show();
            }
        });
        dialog.setCancelable(false);
        dialog.show();
    }

    /**
     * This method is for show ProgressDialog
     *
     * @param show     - this param mean, that if the dialog is to show or not.
     * @param message  - this param used for message to user, what we're loading.
     * @param activity we need the instance of activity for methods which we can call only from activity.
     */
    public static void showProgressDialog(boolean show, String message, final Activity activity) {
        if (show) {
            progressDialog = new ProgressDialog(activity, R.style.AlertDialogCustomStyle);
            progressDialog.getWindow().clearFlags(
                    WindowManager.LayoutParams.FLAG_DIM_BEHIND);
            progressDialog.setMessage(message);
            progressDialog.setCanceledOnTouchOutside(false);
            progressDialog.show();
            progressDialog.setCancelable(false);
        } else {
            if (progressDialog != null && progressDialog.isShowing())
                    progressDialog.dismiss();
        }
    }

    /**
     * Show dialog for decide to get current location or not.
     *
     * @param mMap                     - instance of GoogleMap.
     * @param myLocationChangeListener - LocationChangeListener.
     * @param activity                 - activity.
     */
    public static void showDialogDecideGetLocation(final GoogleMap mMap, final OnMyLocationChangeListener myLocationChangeListener, final Activity activity) {
        AlertDialog.Builder b = new AlertDialog.Builder(activity, R.style.AlertDialogCustomStyle);
        b.setTitle(R.string.dialog_current_location_title);
        b.setMessage(R.string.dialog_current_location_msg);
        b.setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int whichButton) {
                mMap.setOnMyLocationChangeListener(myLocationChangeListener);
            }
        });
        b.setNegativeButton(android.R.string.no, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {

            }
        });
        b.show();
    }

    /**
     * Error dialog.
     *
     * @param title    - error title.
     * @param message  - error message.
     * @param activity - activity for builder.
     */
    public static void showErrorDialog(String title, String message, final Activity activity) {
        AlertDialog.Builder builder = new AlertDialog.Builder(activity, R.style.AlertDialogCustomStyle);
        builder.setTitle(title);
        builder.setMessage(message);
        builder.setIcon(R.drawable.ic_error_uknown);
        builder.setCancelable(false);
        builder.setPositiveButton(R.string.dialog_ok_button, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
            }
        });
        AlertDialog dialog = builder.create();
        dialog.show();
    }


    /**
     * @param title
     * @param message
     * @param activity
     */
    public static void showDecideDialog(String title, String message, final Activity activity, final DialogButtonListener dialogButtonListener) {
        final HistoryFragment historyFragment = new HistoryFragment();
        AlertDialog.Builder builder = new AlertDialog.Builder(activity, R.style.AlertDialogCustomStyle);
        builder.setTitle(title);
        builder.setMessage(message);
        builder.setIcon(R.drawable.ic_info_outline);
        builder.setCancelable(false);
        builder.setPositiveButton(R.string.dialog_ok_button, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialogButtonListener.onButtonClicked(true);
            }
        });
        builder.setNegativeButton(R.string.dialog_cancel_button, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int i) {
                dialogButtonListener.onButtonClicked(false);
            }
        });
        AlertDialog dialog = builder.create();
        dialog.show();
    }

    public interface DialogButtonListener {
        public abstract void onButtonClicked(boolean value);
    }


}
